Package: rversions
Title: Query 'R' Versions, Including 'r-release' and 'r-oldrel'
Version: 3.0.0
Authors@R: 
    c(person(given = "Gábor",
             family = "Csárdi",
             role = c("aut", "cre"),
             email = "csardi.gabor@gmail.com"),
      person(given = "Jeroen",
             family = "Ooms",
             role = "ctb",
             email = "jeroen.ooms@stat.ucla.edu"),
      person(given = "R Consortium",
             role = "fnd"),
      person("Posit Software, PBC", role = c("cph", "fnd"),
             comment = c(ROR = "03wc8by49")))
Description: Query the main 'R' 'SVN' repository to find the
    versions 'r-release' and 'r-oldrel' refer to, and also all previous
    'R' versions and their release dates.
License: MIT + file LICENSE
URL: https://github.com/r-hub/rversions,
        https://r-hub.github.io/rversions/
BugReports: https://github.com/r-hub/rversions/issues
Imports: curl
Suggests: pillar, testthat (>= 3.0.0), webfakes, withr
Encoding: UTF-8
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2025-10-09 13:47:01 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  Jeroen Ooms [ctb],
  R Consortium [fnd],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2025-10-09 14:10:02 UTC
Built: R 4.6.0; ; 2026-04-25 15:58:46 UTC; unix

# Generated by roxygen2: do not edit by hand

export(available)
export(current_r_platform)
export(linux_distros)
export(r_oldrel)
export(r_release)
export(r_release_macos)
export(r_release_tarball)
export(r_release_win)
export(r_versions)
export(resolve)
export(rtools_versions)
importFrom(curl,curl_fetch_memory)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# rversions <a href='https://r-hub.github.io/rversions/'><img src='man/figures/logo.png' align="right" height="138.5" /></a>

> Past and present R versions

<!-- badges: start -->

[![R-CMD-check](https://github.com/r-hub/rversions/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-hub/rversions/actions/workflows/R-CMD-check.yaml)
[![CRAN
downloads](https://cranlogs.r-pkg.org/badges/rversions)](https://r-pkg.org/pkg/rversions)
[![CRAN
version](https://www.r-pkg.org/badges/version/rversions)](https://r-pkg.org/pkg/rversions)
[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![Codecov test
coverage](https://codecov.io/gh/r-hub/rversions/graph/badge.svg)](https://app.codecov.io/gh/r-hub/rversions)
<!-- badges: end -->

The R version numbers and dates are queried from R-hub’s [R version
API](https://api.r-hub.io/rversions/).

`rversions` is in particular useful for determining at any given moment
what versions “R-release” and “R-oldrel” refer to, and what R builds are
available for a given platform.

## Installation

``` r
install.packages("rversions")
```

## Usage

### List all R releases

``` r
r_versions()
```

    #> # A data frame: 161 × 4
    #>    version date                nickname semver    
    #>    <chr>   <dttm>              <chr>    <pckg_vrs>
    #>  1 0.0     1995-06-20 00:00:00 <NA>     0.0.0     
    #>  2 0.1     1996-02-12 03:22:00 <NA>     0.1.0     
    #>  3 0.2     1996-03-14 21:59:00 <NA>     0.2.0     
    #>  4 0.3     1996-03-22 06:46:00 <NA>     0.3.0     
    #>  5 0.4     1996-04-01 23:18:00 <NA>     0.4.0     
    #>  6 0.5     1996-05-13 04:06:00 <NA>     0.5.0     
    #>  7 0.6     1996-05-17 04:53:00 <NA>     0.6.0     
    #>  8 0.7     1996-05-28 03:31:00 <NA>     0.7.0     
    #>  9 0.8     1996-05-31 04:51:00 <NA>     0.8.0     
    #> 10 0.9     1996-06-07 04:40:00 <NA>     0.9.0     
    #> # ℹ 151 more rows

### List available R builds

For the current platform:

``` r
available()
```

    #> # A data frame: 20 × 5
    #>    version date                semver     type    url                           
    #>    <chr>   <dttm>              <pckg_vrs> <chr>   <chr>                         
    #>  1 4.1.0   2021-05-18 07:05:22 4.1.0      release https://cran.rstudio.com/bin/…
    #>  2 4.1.1   2021-08-10 07:05:06 4.1.1      release https://cran.rstudio.com/bin/…
    #>  3 4.1.2   2021-11-01 08:05:12 4.1.2      release https://cran.rstudio.com/bin/…
    #>  4 4.1.3   2022-03-10 08:05:38 4.1.3      release https://cran.rstudio.com/bin/…
    #>  5 4.2.0   2022-04-22 07:05:41 4.2.0      release https://cran.rstudio.com/bin/…
    #>  6 4.2.1   2022-06-23 07:05:33 4.2.1      release https://cran.rstudio.com/bin/…
    #>  7 4.2.2   2022-10-31 08:05:54 4.2.2      release https://cran.rstudio.com/bin/…
    #>  8 4.2.3   2023-03-15 08:06:01 4.2.3      release https://cran.rstudio.com/bin/…
    #>  9 4.3.0   2023-04-21 07:06:14 4.3.0      release https://cran.rstudio.com/bin/…
    #> 10 4.3.1   2023-06-16 07:06:07 4.3.1      release https://cran.rstudio.com/bin/…
    #> 11 4.3.2   2023-10-31 08:07:42 4.3.2      release https://cran.rstudio.com/bin/…
    #> 12 4.3.3   2024-02-29 08:07:53 4.3.3      release https://cran.rstudio.com/bin/…
    #> 13 4.4.0   2024-04-24 04:07:56 4.4.0      release https://cran.rstudio.com/bin/…
    #> 14 4.4.1   2024-06-14 07:08:17 4.4.1      release https://cran.rstudio.com/bin/…
    #> 15 4.4.2   2024-10-31 08:09:02 4.4.2      release https://cran.rstudio.com/bin/…
    #> 16 4.4.3   2025-02-28 08:08:59 4.4.3      release https://cran.rstudio.com/bin/…
    #> 17 4.5.0   2025-04-11 08:51:39 4.5.0      release https://cran.rstudio.com/bin/…
    #> 18 4.5.1   2025-06-13 07:12:01 4.5.1      release https://cran.rstudio.com/bin/…
    #> 19 4.5.1   NA                  4.5.1      patched https://mac.r-project.org/big…
    #> 20 4.6.0   NA                  4.6.0      devel   https://mac.r-project.org/big…

For another platform:

``` r
available("windows")
```

    #> # A data frame: 97 × 5
    #>    version date                semver     type    url                           
    #>    <chr>   <dttm>              <pckg_vrs> <chr>   <chr>                         
    #>  1 2.0.0   2004-10-04 14:24:38 2.0.0      release https://cran-archive.r-projec…
    #>  2 2.0.1   2004-11-15 14:16:30 2.0.1      release https://cran-archive.r-projec…
    #>  3 2.1.0   2005-04-18 22:26:33 2.1.0      release https://cran-archive.r-projec…
    #>  4 2.1.1   2005-06-20 09:27:13 2.1.1      release https://cran-archive.r-projec…
    #>  5 2.2.0   2005-10-06 10:22:14 2.2.0      release https://cran-archive.r-projec…
    #>  6 2.2.1   2005-12-20 10:35:21 2.2.1      release https://cran-archive.r-projec…
    #>  7 2.3.0   2006-04-24 10:37:20 2.3.0      release https://cran-archive.r-projec…
    #>  8 2.3.1   2006-06-01 08:25:33 2.3.1      release https://cran-archive.r-projec…
    #>  9 2.4.0   2006-10-03 10:15:04 2.4.0      release https://cran-archive.r-projec…
    #> 10 2.4.1   2006-12-18 09:49:23 2.4.1      release https://cran-archive.r-projec…
    #> # ℹ 87 more rows

### Resolve R version specifications

Current R release:

``` r
resolve("release")
```

    #> # A data frame: 1 × 6
    #>   version date                nickname          semver     url             type 
    #>   <chr>   <dttm>              <chr>             <pckg_vrs> <chr>           <chr>
    #> 1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1      https://cran.r… rele…

Development version:

``` r
resolve("devel")
```

    #> # A data frame: 1 × 6
    #>   version date   nickname                semver     url                    type 
    #>   <chr>   <dttm> <chr>                   <pckg_vrs> <chr>                  <chr>
    #> 1 4.6.0   NA     Unsuffered Consequences 4.6.0      https://mac.r-project… devel

Next version of R:

``` r
resolve("next")
```

    #> # A data frame: 1 × 6
    #>   version date   nickname          semver     url                          type 
    #>   <chr>   <dttm> <chr>             <pckg_vrs> <chr>                        <chr>
    #> 1 4.5.1   NA     Great Square Root 4.5.1      https://mac.r-project.org/b… next

Previous release:

``` r
resolve("oldrel")
```

    #> # A data frame: 1 × 6
    #>   version date                nickname    semver     url                   type 
    #>   <chr>   <dttm>              <chr>       <pckg_vrs> <chr>                 <chr>
    #> 1 4.4.3   2025-02-28 08:08:59 Trophy Case 4.4.3      https://cran.rstudio… oldr…

Last release of a branch:

``` r
resolve("4.2")
```

    #> # A data frame: 1 × 6
    #>   version date                nickname         semver     url              type 
    #>   <chr>   <dttm>              <chr>            <pckg_vrs> <chr>            <chr>
    #> 1 4.2.3   2023-03-15 08:06:01 Shortstop Beagle 4.2.3      https://cran.rs… 4.2

### List of Rtools versions

``` r
rtools_versions()
```

    #> # A data frame: 21 × 4
    #>    version url                                                       first last 
    #>    <chr>   <chr>                                                     <chr> <chr>
    #>  1 26      https://cran.rstudio.com/bin/windows/Rtools/Rtools26.exe  2.0.0 2.6.2
    #>  2 27      https://cran.rstudio.com/bin/windows/Rtools/Rtools27.exe  2.6.0 2.7.2
    #>  3 28      https://cran.rstudio.com/bin/windows/Rtools/Rtools28.exe  2.7.0 2.8.1
    #>  4 29      https://cran.rstudio.com/bin/windows/Rtools/Rtools29.exe  2.8.0 2.9.2
    #>  5 210     https://cran.rstudio.com/bin/windows/Rtools/Rtools210.exe 2.9.0 2.10…
    #>  6 211     https://cran.rstudio.com/bin/windows/Rtools/Rtools211.exe 2.10… 2.11…
    #>  7 212     https://cran.rstudio.com/bin/windows/Rtools/Rtools212.exe 2.12… 2.12…
    #>  8 213     https://cran.rstudio.com/bin/windows/Rtools/Rtools213.exe 2.13… 2.13…
    #>  9 214     https://cran.rstudio.com/bin/windows/Rtools/Rtools214.exe 2.13… 2.14…
    #> 10 215     https://cran.rstudio.com/bin/windows/Rtools/Rtools215.exe 2.14… 2.15…
    #> # ℹ 11 more rows

### Linux distros supported by Posit, or R-hub

``` r
linux_distros()
```

    #> # A data frame: 42 × 13
    #>    id    aliases name  version codename url   docker eol                 retired
    #>  * <chr> <I<nam> <chr> <chr>   <chr>    <chr> <chr>  <dttm>              <lgl>  
    #>  1 alma… <chr>   Alma… 10      <NA>     http… almal… 2035-05-31 00:00:00 FALSE  
    #>  2 alma… <NULL>  Alma… 8       <NA>     http… almal… 2029-03-01 00:00:00 FALSE  
    #>  3 alma… <chr>   Alma… 9       <NA>     http… almal… 2032-05-31 00:00:00 FALSE  
    #>  4 cent… <chr>   Cent… 6       <NA>     http… cento… 2020-11-30 00:00:00 TRUE   
    #>  5 cent… <chr>   Cent… 7       <NA>     http… cento… 2024-06-30 00:00:00 FALSE  
    #>  6 cent… <NULL>  Cent… 8       <NA>     http… cento… 2024-05-31 00:00:00 FALSE  
    #>  7 debi… <chr>   Debi… 10      buster   http… debia… 2024-06-30 00:00:00 TRUE   
    #>  8 debi… <chr>   Debi… 11      bullseye http… debia… 2024-08-14 00:00:00 TRUE   
    #>  9 debi… <chr>   Debi… 12      bookworm http… debia… NA                  FALSE  
    #> 10 debi… <chr>   Debi… 9       stretch  http… debia… 2022-06-30 00:00:00 TRUE   
    #> # ℹ 32 more rows
    #> # ℹ 4 more variables: aarch64 <chr>, ppm_binaries <lgl>, ppm_aarch64 <lgl>,
    #> #   ppm_url <chr>

## License

MIT © [Gábor Csárdi](https://github.com/gaborcsardi), [Posit Software,
PBC](https://posit.co)

#' Latest R release with a Windows installer available for download
#'
#' [r_release()] works based on the SVN tags in the main R source code
#' repository. However, an SVN tag does not mean that the same version
#' is available for download. `r_release_win` returns the latest
#' version for which a Windows installer is available.
#'
#' @return A one row data frame, with columns \sQuote{version},
#'   \sQuote{date}, \sQuote{URL} and \sQuote{nickname}.
#'
#' @family R downloads
#' @export
#' @importFrom curl new_handle handle_setopt
#' @examplesIf rversions:::is_online()
#' r_release()
#' r_release_win()

r_release_win <- function() {
  keep_head(r_win_download_url(), "-win.exe")
}

r_svn_url <- function() {
  "https://svn.r-project.org/R/tags/"
}

cran_url <- function() {
  if (nzchar(Sys.getenv("R_RVERSIONS_CRAN_URL"))){
    Sys.getenv("R_RVERSIONS_CRAN_URL")
  } else {
    "https://cran.r-project.org"
  }
}

r_download_url <- function() {
  paste0(cran_url(), "/src/base/R-4/")
}

r_win_download_url <- function() {
  paste0(cran_url(), "/bin/windows/base/")
}

r_macos_download_url <- function() {
  paste0(cran_url(), "/bin/macosx/base/")
}

keep_head <- function(base_url, extension) {
  versions <- tail(r_versions(dots = TRUE), 3)
  handle <- new_handle()
  handle_setopt(handle, customrequest = "HEAD")
  handle_setopt(handle, nobody = TRUE)

  for (v in nrow(versions):1) {
    filename <- paste0("R-", versions[v, "version"], extension)
    url <- paste0(base_url, filename)
    resp <- curl_fetch_memory(url, handle = handle)
    if (resp$status_code == 200) {
      df <- versions[v, , drop = FALSE]
      df$URL <- url
      othercols <- setdiff(colnames(df), c("version", "date", "URL"))
      df <- df[, c("version", "date", "URL", othercols)]
      return(df)
    }
  }

  stop("Cannot find R-release version to download", call. = FALSE)
}

cached_nicks <- function() {
  c("0.60" = NA, "0.61" = NA, "0.61.1" = NA,
    "0.61.2" = NA, "0.61.3" = NA, "0.62" = NA,
    "0.62.1" = NA, "0.62.2" = NA, "0.62.3" = NA,
    "0.62.4" = NA, "0.63" = NA, "0.63.1" = NA,
    "0.63.2" = NA, "0.63.3" = NA, "0.64" = NA,
    "0.64.1" = NA, "0.64.2" = NA, "0.65" = NA,
    "0.65.1" = NA, "0.90" = NA, "0.90.1" = NA,
    "0.99" = NA, "1.0" = NA, "1.0.1" = NA, "1.1" = NA,
    "1.1.1" = NA, "1.2" = NA, "1.2.1" = NA, "1.2.2" = NA,
    "1.2.3" = NA, "1.3" = NA, "1.3.1" = NA, "1.4" = NA,
    "1.4.1" = NA, "1.5.0" = NA, "1.5.1" = NA,
    "1.6.0" = NA, "1.6.1" = NA, "1.6.2" = NA,
    "1.7.0" = NA, "1.7.1" = NA, "1.8.0" = NA,
    "1.8.1" = NA, "1.9.0" = NA, "1.9.1" = NA,
    "2.0.0" = NA, "2.0.1" = NA, "2.1.0" = NA,
    "2.1.1" = NA, "2.2.0" = NA, "2.2.1" = NA,
    "2.3.0" = NA, "2.3.1" = NA, "2.4.0" = NA,
    "2.4.1" = NA, "2.5.0" = NA, "2.5.1" = NA,
    "2.6.0" = NA, "2.6.1" = NA, "2.6.2" = NA,
    "2.7.0" = NA, "2.7.1" = NA, "2.7.2" = NA,
    "2.8.0" = NA, "2.8.1" = NA, "2.9.0" = NA,
    "2.9.1" = NA, "2.9.2" = NA, "2.10.0" = NA,
    "2.10.1" = NA, "2.11.0" = NA, "2.11.1" = NA,
    "2.12.0" = NA, "2.12.1" = NA, "2.12.2" = NA,
    "2.13.0" = NA, "2.13.1" = NA, "2.13.2" = NA,
    "2.14.0" = "Great Pumpkin", "2.14.1" = "December Snowflakes",
    "2.14.2" = "Gift-Getting Season",
    "2.15.0" = "Easter Beagle", "2.15.1" = "Roasted Marshmallows",
    "2.15.2" = "Trick or Treat", "2.15.3" = "Security Blanket",
    "3.0.0" = "Masked Marvel", "3.0.1" = "Good Sport",
    "3.0.2" = "Frisbee Sailing", "3.0.3" = "Warm Puppy",
    "3.1.0" = "Spring Dance", "3.1.1" = "Sock it to Me",
    "3.1.2" = "Pumpkin Helmet", "3.1.3" = "Smooth Sidewalk",
    "3.2.0" = "Full of Ingredients", "3.2.1" = "World-Famous Astronaut",
    "3.2.2" = "Fire Safety", "3.2.3" = "Wooden Christmas-Tree",
    "3.2.4" = "Very Secure Dishes", "3.2.5" = "Very, Very Secure Dishes",
    "3.3.0" = "Supposedly Educational", "3.3.1" = "Bug in Your Hair",
    "3.3.2" = "Sincere Pumpkin Patch", "3.3.3" = "Another Canoe",
    "3.4.0" = "You Stupid Darkness", "3.4.1" = "Single Candle",
    "3.4.2" = "Short Summer", "3.4.3" = "Kite-Eating Tree",
    "3.4.4" = "Someone to Lean On", "3.5.0" = "Joy in Playing",
    "3.5.1" = "Feather Spray", "3.5.2" = "Eggshell Igloo",
    "3.5.3" = "Great Truth", "3.6.0" = "Planting of a Tree",
    "3.6.1" = "Action of the Toes", "3.6.2" = "Dark and Stormy Night",
    "3.6.3" = "Holding the Windsock", "4.0.0" = "Arbor Day"
  )
}

get_nicknames <- function(vers = r_versions(dots = TRUE)$version) {
  dash_vers <- gsub(".", "-", vers, fixed = TRUE)
  first_nick <- match("2-15-0", dash_vers)
  if (is.na(first_nick)) first_nick <- 1L
  urls <- sprintf(
    "https://svn.r-project.org/R/tags/R-%s/VERSION-NICK",
    dash_vers[first_nick:length(dash_vers)])

  nicks <- fetch_all(urls)
  res <- rep(NA_character_, length(dash_vers))
  names(res)[first_nick:length(dash_vers)] <- urls
  res[names(nicks)] <- unname(nicks)
  structure(unlist(unname(res)), names = vers)
}

fetch_all <- function(urls) {
  pool <- curl::new_pool(total_con = 100, host_con = 6, multiplex = TRUE)
  results <- list()
  lapply(urls, function(u) {
    h <- curl::new_handle(url = u)
    curl::multi_add(
       h, pool = pool,
       done = function(resp) results[[u]] <<- resp,
       fail = function(err) results[[u]] <<- err)
  })
  curl::multi_run(pool = pool)

  lapply_named(results, get_content)
}

get_content <- function(response, url) {
  if (!is.list(response) || is.null(response$content)) {
    stop("Failed to download ", squote(url))
  }
  str_trim(rawToChar(response$content))
}

lapply_named <- function(X, FUN, ...) {
  mapply(FUN, X, names(X), ..., SIMPLIFY = FALSE)
}

str_trim <- function(x) {
  sub("\\s$", "", sub("^\\s", "", x))
}

squote <- function(x) {
  paste0("'", x, "'")
}

#' @details The R version numbers and dates are extracted from the
#' main R SVN repository at <https://svn.r-project.org/R/>.
#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
## usethis namespace: end
NULL

#' Resolve an R version specification
#'
#' @details
#' `resolve()` caches its results in the current R session. To clear the
#' cache restart R or set the `R_VERSION_CACHE_TIMEOUT` environment
#' variable to `0`.
#'
#' @param version Symbolic version name to resolve. Possible values are
#'   * `release`: the latest release version,
#'   * `devel`: Development version of R.
#'   * `next`: The next version of R. This is R-patched if there is no
#'      ongoing release process currently. Otherwise it may be R-alpha,
#'      R-beta, or R-rc or R-prerelease.
#'   * `oldrel`: The previous version of R, not counting patch versions.
#'   * `oldrel/<n>`: The n-th previous version of R, not counting patch
#'      versions.
#'   * `x.y.z`: A specific R version, e.g. 4.5.1.
#'   * `x.y`: The last patch release from a minor R branch, e.g. 4.4.
#' @param platform Operating system version. Defaults to
#'   [current_r_platform()]. Possible values are:
#'   `r dox_platform_values()`
#'
#' @return A one row data frame, with columns `version`, `date`,
#'   `nickname`, `semver`, `url`, `type`, `rtools_url` (only for Windows).
#' @export
#' @examplesIf rversions:::is_online()
#' resolve("release", "windows")
#' resolve("4.4", "windows-aarch64")
#' resolve("oldrel", "macos")
#' resolve("devel", "linux-gnu-ubuntu-24.04")
#' resolve("devel", "aarch64-unknown-linux-gnu-ubuntu-22.04")

resolve <- function(
  version = "release",
  platform = current_r_platform()
) {
  os <- os_from_platform(platform)
  arch <- arch_from_platform(platform)
  ep <- if (is.na(os)) {
    paste0("/resolve/", version)
  } else {
    paste0("/resolve/", version, "/", os, "/", arch)
  }
  df <- api_version_df1(ep)
  names(df) <- tolower(names(df))
  df
}

# nocov start

dox_platform_values <- function(na = TRUE) {
  paste0(
    collapse = "\n",
    "* ",
    c(
      if (na) {
        "`NA`: to get information about platform-independent release archives."
      },
      "`windows`: x86_64 Windows.",
      "`windows-x86_64`: the same.",
      "`windows-aarch64`: aarch64 Windows.",
      "`macos`: arm64 macOS.",
      "`macos-arm64`: the same.",
      "`macos-x86_64`: x86_64 macOS.",
      "`linux-<distro>-<release>`: x86_64 Linux on the given distribution
     and release, e.g. `linux-ubuntu-22.04`.",
      "A target triplet of the form `cpu-vendor-os-abi`. On Linux it should
     also include the distribution name and release. See
     [current_r_platform()] for examples."
    )
  )
}

# nocov end

test_that("resolve", {
  local_proxy()
  loadNamespace("pillar")
  withr::local_options(width = 500)
  expect_snapshot({
    resolve("release", NA)
  })
  expect_snapshot({
    resolve("devel", "macos")
    resolve("next", "aarch64-apple-darwin20")
  })
  expect_snapshot({
    resolve("oldrel", "x86_64-apple-darwin20")
    resolve("4.3", "macos-x86_64")
  })
  expect_snapshot({
    resolve("release", "linux-gnu-ubuntu-22.04")
    resolve("oldrel", "x86_64-unknown-linux-gnu-ubuntu-24.04")
    resolve("oldrel", "aarch64-unknown-linux-gnu-ubuntu-24.04")
  })
  expect_snapshot({
    resolve("release", "windows")
    resolve("4.4", "x86_64-w64-mingw32")
  })
  expect_snapshot({
    resolve("devel", "windows-aarch64")
    resolve("4.4", "aarch64-w64-mingw32")
  })
})

os_from_platform <- function(platform, na = TRUE) {
  if (na && is.na(platform)) {
    return(NA_character_)
  }

  psd <- parse_platform(platform)

  os <- switch(
    platform,
    "windows" = "windows",
    "windows-x86_64" = "windows",
    "windows-aarch64" = "windows",
    "macos" = "macos",
    "macos-x86_64" = "macos",
    "macos-arm64" = "macos",
    "macos-aarch64" = "macos",
    paste0(
      psd$os,
      if (!is.na(psd$distribution %||% NA)) paste0("-", psd$distribution),
      if (!is.na(psd$release %||% NA)) paste0("-", psd$release)
    )
  )

  os <- if (os == "mingw32") {
    "windows"
  } else if (startsWith(os, "darwin")) {
    "macos"
  } else {
    os
  }
  os
}

arch_from_platform <- function(platform, na = TRUE) {
  if (na && is.na(platform)) {
    return(NA_character_)
  }

  arch <- switch(
    platform,
    "windows" = "x86_64",
    "windows-x86_64" = "x86_64",
    "windows-aarch64" = "aarch64",
    "macos" = "arm64",
    "macos-x86_64" = "x86_64",
    "macos-arm64" = "arm64",
    "macos-aarch64" = "arm64",
    parse_platform(platform)$cpu
  )

  arch
}

# The rest is from pkgcache

#' Current R platform
#'
#' `current_r_platform()` detects the platform of the current R version.
#'
#' It returns a string of the form `cpu-vendor-os-abi`, a target triplet,
#' see some examples below. On Linux it also includes the distribution
#' name and version and part of the ABI.
#'
#' Examples:
#' - `x86_64-apple-darwin17.0`: macOS High Sierra.
#' - `aarch64-apple-darwin20`: macOS Big Sur on arm64.
#' - `x86_64-w64-mingw32`: 64 bit Windows.
#' - `i386-w64-mingw32`: 32 bit Windows.
#' - `i386+x86_64-w64-mingw32`: 64 bit + 32 bit Windows.
#' - `i386-pc-solaris2.10`: 32 bit Solaris. (Some broken 64 Solaris
#'   builds might have the same platform string, unfortunately.)
#' - `x86_64-pc-linux-gnu-debian-10`: Debian Linux 10 on x86_64.
#' - `x86_64-pc-linux-musl-alpine-3.14.1`: Alpine Linux.
#' - `x86_64-pc-linux-gnu-unknown`: Unknown Linux Distribution on x86_64.
#' - `s390x-ibm-linux-gnu-ubuntu-20.04`: Ubuntu Linux 20.04 on S390x.
#' - `amd64-portbld-freebsd12.1`: FreeBSD 12.1 on x86_64.
#'
#' @export
#' @examples
#' current_r_platform()

current_r_platform <- function() {
  current_r_platform_data()$platform
}

current_r_platform_data <- function() {
  forced <- forced_platform()
  if (!is.null(forced)) {
    platform <- parse_platform(forced)
  } else {
    raw <- get_platform(forced = FALSE)
    platform <- parse_platform(raw)
    if (platform$os == "linux" || substr(platform$os, 1, 6) == "linux-") {
      platform <- current_r_platform_data_linux(platform)
    }
  }

  platform$platform <- apply(platform, 1, paste, collapse = "-")
  platform
}

valid_platform_string <- function(x) {
  grepl("^[^-].*[-][^-].*[-][^-].*$", x)
}

forced_platform <- function() {
  opt <- getOption("pkg.current_platform")
  if (!is.null(opt)) {
    if (!is_string(opt)) {
      stop("The `pkg.current_platform` option must be a string scalar.")
    }
    if (!valid_platform_string(opt)) {
      stop(
        "The pkg.current_platform` option must be a valid platform ",
        "triple: `cpu-vendor-os`. \"",
        opt,
        "\" is not."
      )
    }
    return(opt)
  }
  env <- Sys.getenv("PKG_CURRENT_PLATFORM")
  if (env != "") {
    if (is.na(env) || !valid_platform_string(env)) {
      stop(
        "The `PKG_CURRENT_PLATFORM` environment variable must be a valid ",
        "platform triple: \"cpu-vendor-os\". \"",
        env,
        "\" is not."
      )
    }
    return(env)
  }

  NULL
}

get_platform <- function(forced = TRUE) {
  (if (forced) forced_platform()) %||% R.version$platform
}

default_platforms <- function() {
  unique(c(current_r_platform(), "source"))
}

parse_platform <- function(x) {
  # Allow for missing cpu and vendor on Linux
  linux_prefix <- startsWith(x, "linux-")
  x[linux_prefix] <- paste0("x86_64-unknown-", x[linux_prefix])

  pcs <- strsplit(x, "-", fixed = TRUE)
  plt <- data.frame(
    stringsAsFactors = FALSE,
    cpu = vcapply(pcs, "[", 1),
    vendor = vcapply(pcs, "[", 2),
    os = vcapply(pcs, function(y) {
      if (length(y) < 3) NA_character_ else paste(y[-(1:2)], collapse = "-")
    })
  )
  linuxos <- re_match(plt$os, re_linux_platform())
  islinux <- !is.na(linuxos$.match)
  if (any(islinux)) {
    plt$os[islinux] <- linuxos$os[islinux]
    linuxos$distribution[linuxos$distribution == ""] <- NA_character_
    linuxos$release[linuxos$release == ""] <- NA_character_
    plt <- cbind(plt, linuxos[, c("distribution", "release")])
  }
  plt
}

re_linux_platform <- function() {
  paste0(
    "^",
    "(?P<os>linux(?:-gnu|-musl|-uclibc|-dietlibc)?)?",
    "(?:(?:-)(?P<distribution>[^-]+))?",
    "(?:(?:-)(?P<release>.+))?",
    "$"
  )
}

current_r_platform_data_linux <- function(raw, etc = "/etc") {
  os <- tryCatch(
    suppressWarnings(readLines(file.path(etc, "os-release"))),
    error = function(e) NULL
  )
  rh <- tryCatch(
    suppressWarnings(readLines(file.path(etc, "redhat-release"))),
    error = function(e) NULL
  )

  cbind(
    raw[, setdiff(names(raw), c("distribution", "release")), drop = FALSE],
    parse_linux_platform_info(os, rh)
  )
}

unknown_dist <- function() {
  data.frame(
    stringsAsFactors = FALSE,
    distribution = "unknown"
  )
}

parse_linux_platform_info <- function(
  os_release = NULL,
  redhat_release = NULL
) {
  if (
    is.null(os_release) &&
      is.null(redhat_release)
  ) {
    unknown_dist()
  } else if (!is.null(os_release)) {
    parse_os_release(os_release)
  } else {
    parse_redhat_release(redhat_release)
  }
}

is_quoted <- function(x) {
  l <- nchar(x)
  l >= 2 &&
    substr(x, 1, 1) %in% c("'", '"') &&
    substr(x, 1, 1) == substr(x, l, l)
}

remove_quotes <- function(x) {
  l <- nchar(x)
  if (l < 2) {
    x
  } else {
    substr(x, 2, l - 1)
  }
}

parse_os_release <- function(lines) {
  id <- grep("^ID=", lines, value = TRUE)[1]
  if (is.na(id)) {
    return(unknown_dist())
  }
  id <- trimws(sub("^ID=(.*)$", "\\1", id, perl = TRUE))
  if (is_quoted(id)) {
    id <- remove_quotes(id)
  }

  ver <- grep("^VERSION_ID=", lines, value = TRUE)[1]
  if (!is.na(ver)) {
    ver <- trimws(sub("VERSION_ID=(.*)$", "\\1", ver, perl = TRUE))
    if (is_quoted(ver)) ver <- remove_quotes(ver)
  }

  out <- data.frame(
    stringsAsFactors = FALSE,
    distribution = id
  )
  if (!is.na(ver)) {
    out$release <- ver
  }

  if (is.na(ver) && id == "debian") {
    pn <- grep("^PRETTY_NAME=", lines, value = TRUE)[1]
    if (!is.na(pn) && grepl("/sid\"?$", pn)) {
      out$release <- "unstable"
    }
  }

  out
}

parse_redhat_release <- function(lines) {
  pcs <- strsplit(lines[1], " ", fixed = TRUE)[[1]]
  id <- tolower(pcs[1])
  if (id == "" || is.na(id)) {
    return(unknown_dist())
  }

  wver <- grepl("^[-\\.0-9]+$", pcs)

  out <- data.frame(
    stringsAsFactors = FALSE,
    distribution = id
  )
  if (any(wver)) {
    out$release <- pcs[wver][1]
  }

  out
}

test_that("current_r_platform_data", {
  fake(current_r_platform_data, "get_platform", "x86_64-apple-darwin17.0")
  expect_equal(current_r_platform_data()$platform, "x86_64-apple-darwin17.0")
})

test_that("default_platforms", {
  fake(default_platforms, "current_r_platform", "macos")
  expect_equal(default_platforms(), c("macos", "source"))

  fake(default_platforms, "current_r_platform", "windows")
  expect_equal(default_platforms(), c("windows", "source"))

  fake(default_platforms, "current_r_platform", "source")
  expect_equal(default_platforms(), "source")
})

test_that("parse_platform", {
  expect_snapshot({
    parse_platform(c(
      "something-else",
      "aarch64-apple-darwin20",
      "x86_64-w64-mingw32",
      "i386+x86_64-w64-mingw32",
      "aarch64-pc-linux",
      "aarch64-pc-linux-gnu",
      "aarch64-pc-linux-ubuntu",
      "aarch64-pc-linux-ubuntu-22.04",
      "aarch64-pc-linux-ubuntu-22.04-libc++",
      "aarch64-pc-linux-gnu-ubuntu",
      "aarch64-pc-linux-gnu-ubuntu-24.04",
      "aarch64-pc-linux-gnu-ubuntu-24.04-libc++",
      "aarch64-pc-linux-musl-alpine-13.4"
    ))
  })
})

test_that("re_linux_platform", {
  loadNamespace("pillar")
  expect_snapshot({
    re_match(
      c(
        "something-else",
        "linux",
        "linux-gnu",
        "linux-ubuntu",
        "linux-ubuntu-22.04",
        "linux-ubuntu-22.04-libc++",
        "linux-gnu-ubuntu",
        "linux-gnu-ubuntu-24.04",
        "linux-gnu-ubuntu-24.04-libc++",
        "linux-musl-alpine-13.4"
      ),
      re_linux_platform()
    )
  })
})

test_that("current_r_platform_data_linux", {
  dists <- dir(test_path("fixtures", "linux"))
  vers <- lapply(dists, function(d) dir(test_path("fixtures", "linux", d)))

  nlapply <- function(X, FUN, ...) {
    ret <- lapply(X, FUN, ...)
    if (is.character(X) && is.null(names(ret))) {
      names(ret) <- X
    }
    ret
  }

  raw <- data.frame(stringsAsFactors = FALSE, raw = "foo")
  mapply(dists, vers, FUN = function(d, v) {
    etc <- test_path("fixtures", "linux", d, v)
    expect_snapshot(nlapply(etc, current_r_platform_data_linux, raw = raw))
  })
})

test_that("linux", {
  fake(current_r_platform_data, "get_platform", "x86_64-pc-linux-gnu")
  fake(
    current_r_platform_data,
    "current_r_platform_data_linux",
    data.frame(stringsAsFactors = FALSE, x = "boo")
  )
  expect_equal(current_r_platform_data()$platform, "boo")
})

test_that("unknown linux", {
  dummy <- data.frame(stringsAsFactors = FALSE, x = "foo")
  expect_equal(
    current_r_platform_data_linux(dummy, tempfile())$distribution,
    "unknown"
  )
  tmp <- tempfile()
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  dir.create(tmp)
  file.create(file.path(tmp, "os-release"))
  expect_equal(
    current_r_platform_data_linux(dummy, tmp)$distribution,
    "unknown"
  )
})

test_that("remove_quotes", {
  expect_equal(remove_quotes("x"), "x")
  expect_equal(remove_quotes("'xyz'"), "xyz")
  expect_equal(remove_quotes('"xyz"'), "xyz")
})

test_that("parse_redhat_release", {
  expect_equal(parse_redhat_release(""), unknown_dist())
  expect_equal(parse_redhat_release("Something")$distribution, "something")
})

test_that("valid_platform_string", {
  expect_true(valid_platform_string("a-b-c"))
  expect_true(valid_platform_string("a-b-c-"))
  expect_true(valid_platform_string("a-b-c-d"))
  expect_true(valid_platform_string("foo-bar-cup"))
  expect_true(valid_platform_string("foo-bar-cup-boo"))

  expect_false(valid_platform_string("-a-b-c"))
  expect_false(valid_platform_string("a---c"))
  expect_false(valid_platform_string("foo-bar"))
  expect_false(valid_platform_string("foobar"))
})

test_that("option, env var", {
  withr::local_options(pkg.current_platform = "foo-bar-foobar")
  expect_equal(current_r_platform(), "foo-bar-foobar")

  withr::local_options(pkg.current_platform = 1:10)
  expect_snapshot(
    error = TRUE,
    current_r_platform()
  )
  withr::local_options(pkg.current_platform = "foobar")
  expect_snapshot(
    error = TRUE,
    current_r_platform()
  )

  withr::local_options(pkg.current_platform = NULL)
  withr::local_envvar(PKG_CURRENT_PLATFORM = "foobar-foo-bar")
  expect_equal(current_r_platform(), "foobar-foo-bar")

  withr::local_envvar(PKG_CURRENT_PLATFORM = "foobar")
  expect_snapshot(
    error = TRUE,
    current_r_platform()
  )
})

test_that("platform with flavors", {
  withr::local_options(
    pkg.current_platform = "x86_64-pc-linux-gnu-ubuntu-22.04-libc++"
  )
  expect_snapshot(current_r_platform_data())
  expect_snapshot(current_r_platform())
})

test_that("get_platform", {
  fake(get_platform, "forced_platform", "forced")
  expect_equal(get_platform(), "forced")
  expect_equal(get_platform(forced = FALSE), R.version$platform)
})

`%||%` <- function(a, b) {
  if (!is.null(a)) a else b
}

`%|NA|%` <- function(a, b) {
  ifelse(is.na(a), b, a)
}

is_rcmd_check <- function() {
  if (identical(Sys.getenv("NOT_CRAN"), "true")) {
    FALSE
  } else {
    Sys.getenv("_R_CHECK_PACKAGE_NAME_", "") != ""
  }
}

is_online <- function() {
  if (is_rcmd_check()) {
    return(FALSE)
  }
  tolower(Sys.getenv("R_OFFLINE")) != "true"
}

is_string <- function(x) {
  is.character(x) && length(x) == 1 && !is.na(x)
}

vcapply <- function(x, fun, ...) {
  vapply(x, fun, character(1), ...)
}

vlapply <- function(x, fun, ...) {
  vapply(x, fun, logical(1), ...)
}

mkdirp <- function(x) {
  dir.create(x, showWarnings = FALSE, recursive = TRUE)
}

test_that("%||%", {
  expect_equal(NULL %||% 1, 1)
  expect_equal(2 %||% 1, 2)
  expect_equal(1:5 %||% stop("nope"), 1:5)
})

test_that("%|NA|%", {
  expect_equal(NA %|NA|% 1, 1)
  expect_equal(2 %|NA|% 1, 2)
  expect_equal(c(1, NA, 3) %|NA|% 0, c(1, 0, 3))
})

test_that("is_rcmd_check", {
  withr::local_envvar(NOT_CRAN = "true")
  expect_false(is_rcmd_check())
  withr::local_envvar(NOT_CRAN = NA_character_)
  withr::local_envvar(`_R_CHECK_PACKAGE_NAME_` = "rversions")
  expect_true(is_rcmd_check())
  withr::local_envvar(`_R_CHECK_PACKAGE_NAME_` = NA_character_)
  expect_false(is_rcmd_check())
})

test_that("is_online", {
  fake(is_online, "is_rcmd_check", TRUE)
  expect_false(is_online())
  fake(is_online, "is_rcmd_check", FALSE)
  withr::local_envvar(R_OFFLINE = "true")
  expect_false(is_online())
  withr::local_envvar(R_OFFLINE = NA_character_)
  expect_true(is_online())
})

test_that("vcapply", {
  # edge cases
  expect_equal(vcapply(integer(), as.character), character())
  expect_equal(vcapply(1L, as.character), "1")

  expect_equal(vcapply(1:5, as.character), as.character(1:5))

  # names are preserved
  x <- list(a = 1L, b = 2L)
  expect_equal(vcapply(x, as.character), c(a = "1", b = "2"))
})

test_that("vlapply", {
  # edge cases
  expect_equal(vlapply(integer(), is.numeric), logical())
  expect_equal(vlapply(1L, is.numeric), TRUE)

  expect_equal(vlapply(1:5, is.numeric), rep(TRUE, 5))

  # names are preserved
  x <- list(a = 1L, b = "2")
  expect_equal(vlapply(x, is.numeric), c(a = TRUE, b = FALSE))
})

test_that("mkdirp", {
  tmp <- file.path(tempfile(), "foo", "bar")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  mkdirp(tmp)
  expect_true(file.exists(tmp))
  expect_silent(mkdirp(tmp))
})

cache <- new.env(parent = emptyenv())

cache_timeout <- function() {
  as.numeric(Sys.getenv("R_VERSION_CACHE_TIMEOUT", 3600))
}

version_api_url <- function() {
  Sys.getenv("R_VERSION_API_URL", "https://api.r-hub.io/rversions")
}

#' @importFrom curl curl_fetch_memory

api <- function(endpoint) {
  if (
    !is.null(value <- cache[[endpoint]]) &&
      Sys.time() - value$time < cache_timeout()
  ) {
    return(value$value)
  }

  url <- file.path(version_api_url(), endpoint)
  resp <- curl::curl_fetch_memory(url)
  if (resp$status_code != 200) {
    stop("Failed to fetch ", url, "\n", rawToChar(resp$content))
  }

  cache[[endpoint]] <- list(
    time = Sys.time(),
    value = json$parse(rawToChar(resp$content))
  )

  cache[[endpoint]]$value
}

api_version_df <- function(endpoint, dots = TRUE) {
  obj <- api(endpoint)

  obj <- lapply(obj, function(x) {
    x <- as.list(x)
    if (is.null(x$nickname)) {
      x$nickname <- NA_character_
    }
    if (is.null(x$date)) {
      # nocov start
      x$date <- NA_character_
      # nocov end
    }
    x
  })

  df <- list_as_df(obj)
  df <- as_version_df(df, dots)
  df
}

api_version_df1 <- function(endpoint, dots = TRUE) {
  obj <- as.list(api(endpoint))

  if (is.null(obj$nickname)) {
    obj$nickname <- NA_character_
  }
  if (is.null(obj$date)) {
    obj$date <- NA_character_
  }

  df <- list_as_df(list(obj))
  df <- as_version_df(df, dots)
  df
}

null_to_na <- function(x) {
  x[] <- lapply(x, function(y) if (is.null(y)) NA else y)
}

list_as_df <- function(x) {
  x <- lapply(x, null_to_na)
  df <- do.call(rbind.data.frame, x)
  df <- as_data_frame(df)
  df
}

as_version_df <- function(df, dots = TRUE) {
  df$date <- parse_iso_8601(df$date)
  df$version <- to_dots(df$version, dots)
  df$semver <- package_version(df$semver %||% df$version)
  df$URL <- df$URL %||% df$url

  cols <- c(
    "version",
    "date",
    "nickname",
    "semver",
    "URL",
    "type",
    "rtools_url"
  )
  df <- df[intersect(cols, names(df))]

  rownames(df) <- NULL
  df
}

to_dots <- function(x, dots) {
  if (dots) {
    gsub("-", ".", x, fixed = TRUE)
  } else {
    gsub(".", "-", x, fixed = TRUE)
  }
}

#' Latest R release with a macOS installer available for download
#'
#' [r_release()] works based on the SVN tags in the main R source code
#' repository. However, an SVN tag does not mean that the same version
#' is available for download. `r_release_macos` returns the latest
#' version for which a macOS installer is available.
#'
#' @section Note:
#' This function is superseded by [resolve()]:
#' ```{r}
#' r_release_macos()
#' ```
#' ```{r}
#' resolve("release", platform = "macos")
#' ```
#'
#' @param arch The architecture, either \sQuote{x86_64} or \sQuote{arm64}.
#' @return A one row data frame, with columns \sQuote{version},
#'   \sQuote{date}, \sQuote{URL} and \sQuote{nickname}.
#'
#' @family R downloads
#' @export
#' @examplesIf rversions:::is_online()
#' r_release()
#' r_release_macos()
#' r_release_macos(arch = "arm64")

r_release_macos <- function(arch = c("x86_64", "arm64")) {
  arch <- match.arg(arch)
  ep <- switch(
    arch,
    x86_64 = "r-release-macos",
    arm64 = "r-release-macos-arm64",
    stop("Unknown architecture: ", arch)
  )
  df <- api_version_df1(ep)
  df
}

test_that("r_release_macos", {
  local_proxy()
  loadNamespace("pillar")
  withr::local_options(width = 500)
  expect_snapshot({
    r_release_macos("x86_64")
    r_release_macos("arm64")
  })
  expect_snapshot(error = TRUE, {
    r_release_macos("foobar")
  })
})

#' List Rtools versions
#'
#' @param arch Architecture. Possible values are:
#'  * `x86_64`: x86_64 Windows,
#'  * `aarch64` or `arm64`: aarch64 Windows.
#'  Defaults tp the current architecture on Windows, and to `x86_64` on
#'  other systems.
#'
#' @return A data frame with columns
#'   * `version`: The Rtools version, e.g. `45`.
#'   * `url`: Download URL of the Rtools installer.
#'   * `first`: First R version supported by this Rtools version.
#'   * `last`: Last R version supported by this Rtools version.
#'
#' @export
#' @examplesIf rversions:::is_online()
#' rtools_versions()
#' rtools_versions("aarch64")

rtools_versions <- function(arch = NULL) {
  arch <- arch %||% default_windows_arch()
  if (arch == "arm64") {
    arch <- "aarch64"
  }
  ep <- if (arch == "x86_64") {
    "rtools-versions"
  } else {
    paste0("rtools-versions", "/", arch)
  }
  resp <- api(ep)
  list_as_df(resp)
}

default_windows_arch <- function() {
  if (is_windows()) {
    R.version$arch
  } else {
    "x86_64"
  }
}

is_windows <- function() {
  .Platform$OS.type == "windows"
}

test_that("rtools_versions", {
  local_proxy()
  loadNamespace("pillar")
  withr::local_options(width = 500)
  expect_snapshot({
    utils::tail(rtools_versions("x86_64"))
    utils::tail(rtools_versions("aarch64"))
    utils::tail(rtools_versions("arm64"))
  })
})

test_that("default_windows_arch", {
  fake(default_windows_arch, "is_windows", TRUE)
  expect_equal(default_windows_arch(), R.version$arch)
  fake(default_windows_arch, "is_windows", FALSE)
  expect_equal(default_windows_arch(), "x86_64")
})

test_that("is_windows", {
  expect_equal(is_windows(), .Platform$OS.type == "windows")
})

#' Query R's past and present versions
#'
#' R version numbers consist of three numbers (since version 1.4.1):
#' major, minor and patch.
#'
#' We extract the version numbers from the tags in the SVN repository.
#'
#' @param dots Whether to use dots instead of dashes in the version
#'   number.
#' @return A data frame with three columns: \sQuote{version},
#'   \sQuote{date} and \sQuote{nickname}.
#'
#' @export
#' @examplesIf rversions:::is_online()
#' r_versions()

r_versions <- function(dots = TRUE) {
  df <- api_version_df("r-versions", dots)
  df
}

#' Version number of R-release
#'
#' The latest tag in the SVN repository (in terms of version numbers,
#' not dates).
#'
#' @inheritParams r_versions
#' @return A one row data frame, with columns \sQuote{version},
#'   \sQuote{date} and \sQuote{nickname}.
#'
#' @section Note:
#' This function is superseded by [resolve()]:
#' ```{r}
#' r_release()
#' ```
#' ```{r}
#' resolve("release", platform = NA)
#' ```
#'
#' @export
#' @examplesIf rversions:::is_online()
#' r_release()

r_release <- function(dots = TRUE) {
  df <- api_version_df1("resolve/release", dots)
  df
}

#' Version number of R-oldrel
#'
#' R-oldrel is the latest version of the previous minor version.
#' We extract version numbers from the R SVN repository tags.
#'
#' @section Note:
#' This function is superseded by [resolve()]:
#' ```{r}
#' r_oldrel()
#' ```
#' ```{r}
#' resolve("oldrel", platform = NA)
#' ```
#'
#' @inheritParams r_versions
#' @return A one row data frame, with columns \sQuote{version},
#'   \sQuote{date} and \sQuote{nickname}.
#'
#' @export
#' @examplesIf rversions:::is_online()
#' r_oldrel()

r_oldrel <- function(dots = TRUE) {
  df <- api_version_df1("r-oldrel", dots)
  df
}

test_that("versions", {
  local_proxy()
  loadNamespace("pillar")
  d <- r_versions()
  expect_s3_class(d, "data.frame")
  expect_equal(names(d), c("version", "date", "nickname", "semver"))
  expect_true(is.character(d$version))
  expect_s3_class(d$date, "POSIXt")
})

test_that("dots", {
  local_proxy()
  loadNamespace("pillar")
  expect_match(tail(r_versions(FALSE)$version, 1), RE_DASH)
  expect_match(tail(r_versions(TRUE)$version, 1), RE_DOT)
  expect_equal(
    gsub("-", ".", r_versions(FALSE)$version),
    r_versions(TRUE)$version
  )
})

test_that("r_release respects dots", {
  local_proxy()
  loadNamespace("pillar")
  expect_match(r_release(TRUE)$version, RE_DOT)
  expect_match(r_release(FALSE)$version, RE_DASH)
})

test_that("r_oldrel respects dots", {
  local_proxy()
  loadNamespace("pillar")
  expect_match(r_oldrel(TRUE)$version, RE_DOT)
  expect_match(r_oldrel(FALSE)$version, RE_DASH)
})

proxy_app <- function(fixtures) {
  app <- webfakes::new_app()
  app$locals$fixtures <- fixtures
  app$get(webfakes::new_regexp(""), function(req, res) {
    res$set_type("application/json")
    path <- sub("^/", "", req$path)
    fixfile <- paste0(gsub("/", "-", path), ".json")
    fixpath <- file.path(app$locals$fixtures, fixfile)
    if (file.exists(fixpath)) {
      res$send_file(fixfile, root = app$locals$fixtures)
    } else {
      resp <- curl::curl_fetch_memory(
        paste0("https://api.r-hub.io/rversions/", path)
      )
      if (resp$status_code == 200) {
        mkdirp(dirname(fixpath))
        writeBin(resp$content, fixpath)
      }
      res$set_status(resp$status)
      res$send(resp$content)
    }
  })
  app
}

proxy <- webfakes::local_app_process(
  proxy_app(fixtures = testthat::test_path("fixtures/snap"))
)

local_proxy <- function(url = "", .envir = parent.frame()) {
  testthat::skip_on_cran()
  proxy_url <- strip_last_slash(proxy$url())
  withr::local_envvar(R_VERSION_API_URL = proxy_url, .local_envir = .envir)
  invisible(proxy$url(url))
}

strip_last_slash <- function(x) {
  sub("/$", "", x)
}

bad_proxy_app <- function() {
  app <- webfakes::new_app()
  app$get(webfakes::new_regexp(""), function(req, res) {
    res$set_status(404)
    res$set_type("application/json")
    res$send_json(list(error = "Internal server error"))
  })
  app
}

test_that("available, macos", {
  local_proxy()
  loadNamespace("pillar")
  withr::local_options(width = 200)
  expect_snapshot(
    available("macos")
  )
})

test_that("available, macos-x86_64", {
  local_proxy()
  loadNamespace("pillar")
  withr::local_options(width = 200)
  expect_snapshot(
    available("macos-x86_64")
  )
})

test_that("available, windows", {
  local_proxy()
  loadNamespace("pillar")
  withr::local_options(width = 200)
  expect_snapshot(
    utils::tail(available("windows"))
  )
})

test_that("available, windows-aarch64", {
  local_proxy()
  loadNamespace("pillar")
  withr::local_options(width = 200)
  expect_snapshot(
    available("windows-aarch64")
  )
})

test_that("available, ubuntu", {
  local_proxy()
  loadNamespace("pillar")
  withr::local_options(width = 200)
  expect_snapshot(
    utils::tail(available("linux-ubuntu-24.04"))
  )
})

test_that("available, ubuntu aarch64", {
  local_proxy()
  loadNamespace("pillar")
  withr::local_options(width = 200)
  expect_snapshot(
    utils::tail(available("aarch64-unknown-linux-ubuntu-24.04"))
  )
})

test_that("api errors", {
  skip_on_cran()
  proxy <- webfakes::local_app_process(bad_proxy_app())
  withr::local_envvar(R_VERSION_API_URL = proxy$url())
  expect_snapshot(error = TRUE, transform = redact_port, {
    r_versions()
  })
})

test_that("version without nickname", {
  skip_on_cran()
  local_proxy()
  loadNamespace("pillar")
  withr::local_options(width = 500)
  expect_snapshot({
    resolve("2.0.0", platform = NA)
  })
})

#' Return information about Posit's (and some other) R builds for Linux
#'
#' @details
#' `linux_distros()` caches its results in the current R session. To clear
#' the cache restart R or set the `R_VERSION_CACHE_TIMEOUT` environment
#' variable to `0`.
#'
#' @return A data frame with the following columns:
#' * `id`: The distro ID, e.g. `ubuntu-2404`.
#' * `aliases`: A list column of character vectors with alternative names
#'   for the distro. Alternative names starting and ending with a forward
#'   slash are regular expressions.
#' * `name`: The full name of the distro, e.g. `Red Hat Enterprise Linux`.
#' * `version`: The version of the distro, e.g. `24.04` for Ubuntu, or `9`
#'   for RHEL.
#' * `codename`: The codename of the distro, e.g. `focal` for Ubuntu 20.04.
#' * `url`: The URL of the distro's homepage.
#' * `docker`: The name of the official Docker image for the distro,
#'   e.g. `ubuntu:20.04`.
#' * `eol`: The end-of-life date of the distro, or `NA` if unknown.
#' * `retired`: whether the distro version is retired for Posit and R-hub.
#' * `aarch64`: whether aarch64 builds are available for this distro.
#'    Possible values are `posit` (builds are available from Posit),
#'    `r-hub` (builds are available from R-hub), `none` (no builds are
#'    available).
#' * `ppm_binaries`: whether PPM binaries are available for this distro.
#' * `ppm_aarch64`: whether aarch64 PPM binaries are available for this
#'   distro.
#' * `ppm_url`: The URL of the PPM binaries for this distro.
#'
#' @export
#' @examplesIf rversions:::is_online()
#' linux_distros()

linux_distros <- function() {
  res <- api("/linux-distros")

  # Some distros are implemented as aliases of others
  bas <- Filter(function(x) is.null(x$implementation), res)
  names(bas) <- vcapply(bas, `[[`, "id")
  ali <- Filter(function(x) !is.null(x$implementation), res)
  for (i in seq_along(ali)) {
    ali[[i]] <- utils::modifyList(bas[[ali[[i]]$implementation]], ali[[i]])
  }
  res <- c(bas, ali)

  xc <- function(name) {
    unname(vcapply(res, function(x) x[[name]] %||% NA_character_))
  }

  xl <- function(name) {
    unname(vlapply(res, function(x) x[[name]] %||% NA))
  }

  eol <- xc("eol")
  eol[eol == "N/A"] <- NA_character_

  df <- data_frame(
    id = xc("id"),
    aliases = I(lapply(res, function(x) unlist(x$aliases))),
    name = xc("name"),
    version = xc("version"),
    codename = xc("codename"),
    url = xc("url"),
    docker = xc("docker"),
    eol = parse_iso_8601(eol),
    retired = xl("retired"),
    aarch64 = unname(vcapply(res, function(x) {
      if (isTRUE(x$aarch64)) {
        "posit"
      } else if (isFALSE(x$aarch64)) {
        "none"
      } else {
        x$aarch64
      }
    })),
    ppm_binaries = xl("ppm-binaries"),
    ppm_aarch64 = xl("aarch64-ppm-binaries"),
    ppm_url = xc("ppm-binary-url")
  )

  df <- df[order(df$id), ]

  df
}

#' Available R Builds
#'
#' @details
#' `available()` caches its results in the current R session. To clear the
#' cache restart R or set the `R_VERSION_CACHE_TIMEOUT` environment
#' variable to `0`.
#'
#' @param platform Operating system version. Defaults to
#'   [current_r_platform()]. Possible values are:
#'   `r dox_platform_values(na = FALSE)`
#'
#' @return A data frame with columns `version`, `date`, `semver`, `type`,
#'   and `url`.
#'
#' @export
#' @examplesIf rversions:::is_online()
#' available()
#' available("windows")
#' available("macos-arm64")
#' available("linux-gnu-ubuntu-24.04")
#' available("aarch64-unknown-linux-gnu-ubuntu-22.04")

available <- function(platform = current_r_platform()) {
  os <- os_from_platform(platform, na = FALSE)
  arch <- arch_from_platform(platform, na = FALSE)
  ep <- paste0("available/", os, "/", arch)
  resp <- api(ep)
  cols <- c("version", "date", "semver", "type", "url")
  resp <- lapply(resp, function(x) structure(x[cols], names = cols))
  df <- list_as_df(resp)
  df$date <- parse_iso_8601(df$date)
  df$semver <- package_version(df$semver %|NA|% df$version)
  names(df) <- tolower(names(df))
  df
}

# r_release_tarball

    Code
      r_release_tarball()
    Output
      # A data frame: 1 x 5
        version date                nickname          semver     URL                                                 
        <chr>   <dttm>              <chr>             <pckg_vrs> <chr>                                               
      1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1      https://cran.rstudio.com/src/base/R-4/R-4.5.1.tar.gz

# parse_platform

    Code
      parse_platform(c("something-else", "aarch64-apple-darwin20",
        "x86_64-w64-mingw32", "i386+x86_64-w64-mingw32", "aarch64-pc-linux",
        "aarch64-pc-linux-gnu", "aarch64-pc-linux-ubuntu",
        "aarch64-pc-linux-ubuntu-22.04", "aarch64-pc-linux-ubuntu-22.04-libc++",
        "aarch64-pc-linux-gnu-ubuntu", "aarch64-pc-linux-gnu-ubuntu-24.04",
        "aarch64-pc-linux-gnu-ubuntu-24.04-libc++",
        "aarch64-pc-linux-musl-alpine-13.4"))
    Output
                 cpu vendor         os distribution      release
      1    something   else       <NA>         <NA>         <NA>
      2      aarch64  apple   darwin20         <NA>         <NA>
      3       x86_64    w64    mingw32         <NA>         <NA>
      4  i386+x86_64    w64    mingw32         <NA>         <NA>
      5      aarch64     pc      linux         <NA>         <NA>
      6      aarch64     pc  linux-gnu         <NA>         <NA>
      7      aarch64     pc      linux       ubuntu         <NA>
      8      aarch64     pc      linux       ubuntu        22.04
      9      aarch64     pc      linux       ubuntu 22.04-libc++
      10     aarch64     pc  linux-gnu       ubuntu         <NA>
      11     aarch64     pc  linux-gnu       ubuntu        24.04
      12     aarch64     pc  linux-gnu       ubuntu 24.04-libc++
      13     aarch64     pc linux-musl       alpine         13.4

# re_linux_platform

    Code
      re_match(c("something-else", "linux", "linux-gnu", "linux-ubuntu",
        "linux-ubuntu-22.04", "linux-ubuntu-22.04-libc++", "linux-gnu-ubuntu",
        "linux-gnu-ubuntu-24.04", "linux-gnu-ubuntu-24.04-libc++",
        "linux-musl-alpine-13.4"), re_linux_platform())
    Output
      # A data frame: 10 x 5
         os         distribution release        .text                         .match  
         <chr>      <chr>        <chr>          <chr>                         <chr>   
       1 <NA>        <NA>         <NA>          something-else                <NA>    
       2 linux      ""           ""             linux                         linux   
       3 linux-gnu  ""           ""             linux-gnu                     linux-g~
       4 linux      "ubuntu"     ""             linux-ubuntu                  linux-u~
       5 linux      "ubuntu"     "22.04"        linux-ubuntu-22.04            linux-u~
       6 linux      "ubuntu"     "22.04-libc++" linux-ubuntu-22.04-libc++     linux-u~
       7 linux-gnu  "ubuntu"     ""             linux-gnu-ubuntu              linux-g~
       8 linux-gnu  "ubuntu"     "24.04"        linux-gnu-ubuntu-24.04        linux-g~
       9 linux-gnu  "ubuntu"     "24.04-libc++" linux-gnu-ubuntu-24.04-libc++ linux-g~
      10 linux-musl "alpine"     "13.4"         linux-musl-alpine-13.4        linux-m~

# current_r_platform_data_linux

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/almalinux/8`
        raw distribution release
      1 foo    almalinux    8.10
      
      $`fixtures/linux/almalinux/9`
        raw distribution release
      1 foo    almalinux     9.5
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/alpine/3.11`
        raw distribution release
      1 foo       alpine 3.11.11
      
      $`fixtures/linux/alpine/3.12`
        raw distribution release
      1 foo       alpine  3.12.7
      
      $`fixtures/linux/alpine/3.13`
        raw distribution release
      1 foo       alpine  3.13.5
      
      $`fixtures/linux/alpine/3.14`
        raw distribution release
      1 foo       alpine  3.14.1
      
      $`fixtures/linux/alpine/3.18`
        raw distribution release
      1 foo       alpine 3.18.12
      
      $`fixtures/linux/alpine/3.19`
        raw distribution release
      1 foo       alpine  3.19.7
      
      $`fixtures/linux/alpine/3.20`
        raw distribution release
      1 foo       alpine  3.20.6
      
      $`fixtures/linux/alpine/3.21`
        raw distribution release
      1 foo       alpine  3.21.3
      
      $`fixtures/linux/alpine/edge`
        raw distribution              release
      1 foo       alpine 3.22.0_alpha20250108
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/arch/base`
        raw distribution           release
      1 foo         arch 20250511.0.348143
      
      $`fixtures/linux/arch/base-devel`
        raw distribution           release
      1 foo         arch 20250511.0.348143
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/centos/5`
        raw distribution release
      1 foo       centos    5.11
      
      $`fixtures/linux/centos/6`
        raw distribution release
      1 foo       centos    6.10
      
      $`fixtures/linux/centos/7`
        raw distribution release
      1 foo       centos       7
      
      $`fixtures/linux/centos/8`
        raw distribution release
      1 foo       centos       8
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/debian/10`
        raw distribution release
      1 foo       debian      10
      
      $`fixtures/linux/debian/11`
        raw distribution release
      1 foo       debian      11
      
      $`fixtures/linux/debian/12`
        raw distribution release
      1 foo       debian      12
      
      $`fixtures/linux/debian/8`
        raw distribution release
      1 foo       debian       8
      
      $`fixtures/linux/debian/9`
        raw distribution release
      1 foo       debian       9
      
      $`fixtures/linux/debian/testing`
        raw distribution  release
      1 foo       debian unstable
      
      $`fixtures/linux/debian/unstable`
        raw distribution  release
      1 foo       debian unstable
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/fedora/32`
        raw distribution release
      1 foo       fedora      32
      
      $`fixtures/linux/fedora/33`
        raw distribution release
      1 foo       fedora      33
      
      $`fixtures/linux/fedora/34`
        raw distribution release
      1 foo       fedora      34
      
      $`fixtures/linux/fedora/35`
        raw distribution release
      1 foo       fedora      35
      
      $`fixtures/linux/fedora/38`
        raw distribution release
      1 foo       fedora      38
      
      $`fixtures/linux/fedora/39`
        raw distribution release
      1 foo       fedora      39
      
      $`fixtures/linux/fedora/40`
        raw distribution release
      1 foo       fedora      40
      
      $`fixtures/linux/fedora/41`
        raw distribution release
      1 foo       fedora      41
      
      $`fixtures/linux/fedora/42`
        raw distribution release
      1 foo       fedora      42
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/opensuse/15.0`
        raw  distribution release
      1 foo opensuse-leap    15.0
      
      $`fixtures/linux/opensuse/15.1`
        raw  distribution release
      1 foo opensuse-leap    15.1
      
      $`fixtures/linux/opensuse/15.2`
        raw  distribution release
      1 foo opensuse-leap    15.2
      
      $`fixtures/linux/opensuse/15.3`
        raw  distribution release
      1 foo opensuse-leap    15.3
      
      $`fixtures/linux/opensuse/15.4`
        raw  distribution release
      1 foo opensuse-leap    15.4
      
      $`fixtures/linux/opensuse/15.5`
        raw  distribution release
      1 foo opensuse-leap    15.5
      
      $`fixtures/linux/opensuse/15.6`
        raw  distribution release
      1 foo opensuse-leap    15.6
      
      $`fixtures/linux/opensuse/42.3`
        raw distribution release
      1 foo     opensuse    42.3
      
      $`fixtures/linux/opensuse/tumbleweed`
        raw        distribution  release
      1 foo opensuse-tumbleweed 20210810
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/oraclelinux/6`
        raw distribution release
      1 foo           ol    6.10
      
      $`fixtures/linux/oraclelinux/7`
        raw distribution release
      1 foo           ol     7.9
      
      $`fixtures/linux/oraclelinux/8`
        raw distribution release
      1 foo           ol     8.4
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/rhel/7`
        raw distribution release
      1 foo         rhel     7.9
      
      $`fixtures/linux/rhel/8`
        raw distribution release
      1 foo         rhel    8.10
      
      $`fixtures/linux/rhel/9`
        raw distribution release
      1 foo         rhel     9.6
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/rocky/8`
        raw distribution release
      1 foo        rocky     8.9
      
      $`fixtures/linux/rocky/9`
        raw distribution release
      1 foo        rocky     9.3
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/sl/6`
        raw distribution release
      1 foo   scientific    6.10
      
      $`fixtures/linux/sl/7`
        raw distribution release
      1 foo   scientific     7.9
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/sles/15.3`
        raw distribution release
      1 foo         sles    15.3
      
      $`fixtures/linux/sles/15.4`
        raw distribution release
      1 foo         sles    15.4
      
      $`fixtures/linux/sles/15.5`
        raw distribution release
      1 foo         sles    15.5
      
      $`fixtures/linux/sles/15.6`
        raw distribution release
      1 foo         sles    15.6
      

---

    Code
      nlapply(etc, current_r_platform_data_linux, raw = raw)
    Output
      $`fixtures/linux/ubuntu/14.04`
        raw distribution release
      1 foo       ubuntu   14.04
      
      $`fixtures/linux/ubuntu/16.04`
        raw distribution release
      1 foo       ubuntu   16.04
      
      $`fixtures/linux/ubuntu/18.04`
        raw distribution release
      1 foo       ubuntu   18.04
      
      $`fixtures/linux/ubuntu/20.04`
        raw distribution release
      1 foo       ubuntu   20.04
      
      $`fixtures/linux/ubuntu/21.04`
        raw distribution release
      1 foo       ubuntu   21.04
      
      $`fixtures/linux/ubuntu/21.10`
        raw distribution release
      1 foo       ubuntu   21.10
      
      $`fixtures/linux/ubuntu/22.04`
        raw distribution release
      1 foo       ubuntu   22.04
      
      $`fixtures/linux/ubuntu/22.10`
        raw distribution release
      1 foo       ubuntu   22.10
      
      $`fixtures/linux/ubuntu/24.04`
        raw distribution release
      1 foo       ubuntu   24.04
      

# option, env var

    Code
      current_r_platform()
    Condition
      Error in `forced_platform()`:
      ! The `pkg.current_platform` option must be a string scalar.

---

    Code
      current_r_platform()
    Condition
      Error in `forced_platform()`:
      ! The pkg.current_platform` option must be a valid platform triple: `cpu-vendor-os`. "foobar" is not.

---

    Code
      current_r_platform()
    Condition
      Error in `forced_platform()`:
      ! The `PKG_CURRENT_PLATFORM` environment variable must be a valid platform triple: "cpu-vendor-os". "foobar" is not.

# platform with flavors

    Code
      current_r_platform_data()
    Output
           cpu vendor        os distribution      release
      1 x86_64     pc linux-gnu       ubuntu 22.04-libc++
                                       platform
      1 x86_64-pc-linux-gnu-ubuntu-22.04-libc++

---

    Code
      current_r_platform()
    Output
      [1] "x86_64-pc-linux-gnu-ubuntu-22.04-libc++"

# r_release_win

    Code
      r_release_win()
    Output
      # A data frame: 1 x 5
        version date                nickname          semver     URL                                                      
        <chr>   <dttm>              <chr>             <pckg_vrs> <chr>                                                    
      1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1      https://cran.rstudio.com/bin/windows/base/R-4.5.1-win.exe

# resolve

    Code
      resolve("release", NA)
    Output
      # A data frame: 1 x 6
        version date                nickname          semver     url                                                  type   
        <chr>   <dttm>              <chr>             <pckg_vrs> <chr>                                                <chr>  
      1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1      https://cran.rstudio.com/src/base/R-4/R-4.5.1.tar.gz release

---

    Code
      resolve("devel", "macos")
    Output
      # A data frame: 1 x 6
        version date   nickname                semver     url                                                              type 
        <chr>   <dttm> <chr>                   <pckg_vrs> <chr>                                                            <chr>
      1 4.6.0   NA     Unsuffered Consequences 4.6.0      https://mac.r-project.org/big-sur/last-success/R-devel-arm64.pkg devel
    Code
      resolve("next", "aarch64-apple-darwin20")
    Output
      # A data frame: 1 x 6
        version date   nickname          semver     url                                                                   type 
        <chr>   <dttm> <chr>             <pckg_vrs> <chr>                                                                 <chr>
      1 4.5.1   NA     Great Square Root 4.5.1      https://mac.r-project.org/big-sur/last-success/R-4.5-branch-arm64.pkg next 

---

    Code
      resolve("oldrel", "x86_64-apple-darwin20")
    Output
      # A data frame: 1 x 6
        version date                nickname    semver     url                                                                        type    
        <chr>   <dttm>              <chr>       <pckg_vrs> <chr>                                                                      <chr>   
      1 4.4.3   2025-02-28 08:08:59 Trophy Case 4.4.3      https://cran.rstudio.com/bin/macosx/big-sur-x86_64/base/R-4.4.3-x86_64.pkg oldrel/1
    Code
      resolve("4.3", "macos-x86_64")
    Output
      # A data frame: 1 x 6
        version date                nickname        semver     url                                                                        type 
        <chr>   <dttm>              <chr>           <pckg_vrs> <chr>                                                                      <chr>
      1 4.3.3   2024-02-29 08:07:53 Angel Food Cake 4.3.3      https://cran.rstudio.com/bin/macosx/big-sur-x86_64/base/R-4.3.3-x86_64.pkg 4.3  

---

    Code
      resolve("release", "linux-gnu-ubuntu-22.04")
    Output
      # A data frame: 1 x 6
        version date                nickname          semver     url                                                         type   
        <chr>   <dttm>              <chr>             <pckg_vrs> <chr>                                                       <chr>  
      1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1      https://cdn.posit.co/r/ubuntu-2204/pkgs/r-4.5.1_1_amd64.deb release
    Code
      resolve("oldrel", "x86_64-unknown-linux-gnu-ubuntu-24.04")
    Output
      # A data frame: 1 x 6
        version date                nickname    semver     url                                                         type    
        <chr>   <dttm>              <chr>       <pckg_vrs> <chr>                                                       <chr>   
      1 4.4.3   2025-02-28 08:08:59 Trophy Case 4.4.3      https://cdn.posit.co/r/ubuntu-2404/pkgs/r-4.4.3_1_amd64.deb oldrel/1
    Code
      resolve("oldrel", "aarch64-unknown-linux-gnu-ubuntu-24.04")
    Output
      # A data frame: 1 x 6
        version date                nickname    semver     url                                                         type    
        <chr>   <dttm>              <chr>       <pckg_vrs> <chr>                                                       <chr>   
      1 4.4.3   2025-02-28 08:08:59 Trophy Case 4.4.3      https://cdn.posit.co/r/ubuntu-2404/pkgs/r-4.4.3_1_arm64.deb oldrel/1

---

    Code
      resolve("release", "windows")
    Output
      # A data frame: 1 x 7
        version date                nickname          semver     url                                                       type    rtools_url                                                             
        <chr>   <dttm>              <chr>             <pckg_vrs> <chr>                                                     <chr>   <chr>                                                                  
      1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1      https://cran.rstudio.com/bin/windows/base/R-4.5.1-win.exe release https://github.com/r-hub/rtools45/releases/download/latest/rtools45.exe
    Code
      resolve("4.4", "x86_64-w64-mingw32")
    Output
      # A data frame: 1 x 7
        version date                nickname    semver     url                                                                 type  rtools_url                                                             
        <chr>   <dttm>              <chr>       <pckg_vrs> <chr>                                                               <chr> <chr>                                                                  
      1 4.4.3   2025-02-28 08:08:59 Trophy Case 4.4.3      https://cran.rstudio.com/bin/windows/base/old/4.4.3/R-4.4.3-win.exe 4.4   https://github.com/r-hub/rtools44/releases/download/latest/rtools44.exe

---

    Code
      resolve("devel", "windows-aarch64")
    Output
      # A data frame: 1 x 7
        version date   nickname                semver     url                                                                                       type  rtools_url                                                                     
        <chr>   <dttm> <chr>                   <pckg_vrs> <chr>                                                                                     <chr> <chr>                                                                          
      1 4.6.0   NA     Unsuffered Consequences 4.6.0      https://github.com/r-hub/R/releases/download/vdevel/R-devel-2025-10-06-r88901-aarch64.exe devel https://github.com/r-hub/rtools45/releases/download/latest/rtools45-aarch64.exe
    Code
      resolve("4.4", "aarch64-w64-mingw32")
    Output
      # A data frame: 1 x 7
        version date                nickname    semver     url                                                                     type  rtools_url                                                                     
        <chr>   <dttm>              <chr>       <pckg_vrs> <chr>                                                                   <chr> <chr>                                                                          
      1 4.4.3   2025-02-28 08:08:59 Trophy Case 4.4.3      https://github.com/r-hub/R/releases/download/v4.4.3/R-4.4.3-aarch64.exe 4.4   https://github.com/r-hub/rtools44/releases/download/latest/rtools44-aarch64.exe

# linux_distros

    Code
      print(linux_distros(), n = Inf)
    Output
      # A data frame: 42 x 13
         id           aliases         name                         version codename        url                        docker                              eol                 retired aarch64 ppm_binaries ppm_aarch64 ppm_url    
       * <chr>        <I<named list>> <chr>                        <chr>   <chr>           <chr>                      <chr>                               <dttm>              <lgl>   <chr>   <lgl>        <lgl>       <chr>      
       1 almalinux-10 <chr [1]>       AlmaLinux                    10      <NA>            https://almalinux.org/     almalinux:10                        2035-05-31 00:00:00 FALSE   posit   TRUE         TRUE        rhel10     
       2 almalinux-8  <NULL>          AlmaLinux                    8       <NA>            https://almalinux.org/     almalinux:8                         2029-03-01 00:00:00 FALSE   posit   TRUE         NA          centos8    
       3 almalinux-9  <chr [1]>       AlmaLinux                    9       <NA>            https://almalinux.org/     almalinux:9                         2032-05-31 00:00:00 FALSE   posit   TRUE         TRUE        rhel9      
       4 centos-6     <chr [1]>       CentOS Linux                 6       <NA>            https://www.centos.org/    centos:6                            2020-11-30 00:00:00 TRUE    none    FALSE        NA          <NA>       
       5 centos-7     <chr [1]>       CentOS Linux                 7       <NA>            https://www.centos.org/    centos:7                            2024-06-30 00:00:00 FALSE   posit   TRUE         NA          centos7    
       6 centos-8     <NULL>          CentOS Linux                 8       <NA>            https://www.centos.org/    centos:8                            2024-05-31 00:00:00 FALSE   posit   TRUE         NA          centos8    
       7 debian-10    <chr [1]>       Debian                       10      buster          https://www.debian.org/    debian:10                           2024-06-30 00:00:00 TRUE    rhub    FALSE        NA          <NA>       
       8 debian-11    <chr [1]>       Debian                       11      bullseye        https://www.debian.org/    debian:11                           2024-08-14 00:00:00 TRUE    rhub    TRUE         NA          bullseye   
       9 debian-12    <chr [1]>       Debian                       12      bookworm        https://www.debian.org/    debian:12                           NA                  FALSE   posit   TRUE         NA          bookworm   
      10 debian-9     <chr [1]>       Debian                       9       stretch         https://www.debian.org/    debian:9                            2022-06-30 00:00:00 TRUE    rhub    FALSE        NA          <NA>       
      11 fedora-37    <NULL>          Fedora Linux                 37      <NA>            https://fedoraproject.org/ fedora:37                           2023-11-14 00:00:00 TRUE    rhub    FALSE        NA          <NA>       
      12 fedora-38    <NULL>          Fedora Linux                 38      <NA>            https://fedoraproject.org/ fedora:38                           2024-05-14 00:00:00 TRUE    rhub    FALSE        NA          <NA>       
      13 fedora-39    <NULL>          Fedora Linux                 39      <NA>            https://fedoraproject.org/ fedora:39                           2024-11-12 00:00:00 TRUE    rhub    FALSE        NA          <NA>       
      14 fedora-40    <NULL>          Fedora Linux                 40      <NA>            https://fedoraproject.org/ fedora:40                           2025-05-13 00:00:00 TRUE    posit   FALSE        NA          <NA>       
      15 fedora-41    <NULL>          Fedora Linux                 41      <NA>            https://fedoraproject.org/ fedora:41                           2025-11-26 00:00:00 FALSE   posit   FALSE        NA          <NA>       
      16 fedora-42    <NULL>          Fedora Linux                 42      <NA>            https://fedoraproject.org/ fedora:42                           2026-05-13 00:00:00 FALSE   posit   FALSE        NA          <NA>       
      17 opensuse-15  <chr [6]>       openSUSE Leap                15.1    <NA>            https://www.opensuse.org/  opensuse/leap:15.1                  2021-02-01 00:00:00 TRUE    none    TRUE         NA          opensuse15 
      18 opensuse-152 <chr [2]>       openSUSE Leap                15.2    <NA>            https://www.opensuse.org/  opensuse/leap:15.2                  2022-01-04 00:00:00 TRUE    none    TRUE         NA          opensuse152
      19 opensuse-153 <chr [2]>       openSUSE Leap                15.3    <NA>            https://www.opensuse.org/  opensuse/leap:15.3                  2022-12-31 00:00:00 TRUE    rhub    TRUE         NA          opensuse153
      20 opensuse-154 <chr [2]>       openSUSE Leap                15.4    <NA>            https://www.opensuse.org/  opensuse/leap:15.4                  2023-12-31 00:00:00 TRUE    rhub    TRUE         NA          opensuse154
      21 opensuse-155 <chr [2]>       openSUSE Leap                15.5    <NA>            https://www.opensuse.org/  opensuse/leap:15.5                  2024-12-31 00:00:00 TRUE    rhub    TRUE         NA          opensuse155
      22 opensuse-156 <chr [2]>       openSUSE Leap                15.6    <NA>            https://www.opensuse.org/  opensuse/leap:15.6                  2025-12-31 00:00:00 FALSE   posit   TRUE         NA          opensuse156
      23 opensuse-42  <chr [2]>       openSUSE Leap                42.3    <NA>            https://www.opensuse.org/  opensuse/archive:42.3               2019-07-01 00:00:00 TRUE    none    TRUE         NA          opensuse42 
      24 rhel-10      <chr [1]>       Red Hat Enterprise Linux     10      <NA>            https://www.redhat.com/    redhat/ubi10                        2035-05-31 00:00:00 FALSE   posit   TRUE         TRUE        rhel10     
      25 rhel-7       <chr [1]>       Red Hat Enterprise Linux     7       <NA>            https://www.redhat.com/    registry.access.redhat.com/ubi7/ubi 2024-06-30 00:00:00 FALSE   posit   TRUE         NA          centos7    
      26 rhel-8       <NULL>          Red Hat Enterprise Linux     8       <NA>            https://www.redhat.com/    redhat/ubi8                         2029-05-31 00:00:00 FALSE   posit   TRUE         NA          centos8    
      27 rhel-9       <chr [1]>       Red Hat Enterprise Linux     9       <NA>            https://www.redhat.com/    redhat/ubi9                         2032-05-31 00:00:00 FALSE   posit   TRUE         TRUE        rhel9      
      28 rocky-10     <chr [1]>       Rocky Linux                  10      <NA>            https://rockylinux.org/    rockylinux/rockylinux:10            2035-05-31 00:00:00 FALSE   posit   TRUE         TRUE        rhel10     
      29 rocky-8      <NULL>          Rocky Linux                  8       <NA>            https://rockylinux.org/    rockylinux/rockylinux:8             2029-05-31 00:00:00 FALSE   posit   TRUE         NA          centos8    
      30 rocky-9      <chr [1]>       Rocky Linux                  9       <NA>            https://rockylinux.org/    rockylinux/rockylinux:9             2032-05-31 00:00:00 FALSE   posit   TRUE         TRUE        rhel9      
      31 sles-15      <chr [6]>       SUSE Linux Enterprise Server 15      <NA>            https://suse.com           registry.suse.com/suse/sle15:15.0   2019-12-31 00:00:00 TRUE    none    TRUE         NA          opensuse15 
      32 sles-15.1    <chr [6]>       SUSE Linux Enterprise Server 15.1    <NA>            https://suse.com           registry.suse.com/suse/sle15:15.1   2021-12-31 00:00:00 TRUE    none    TRUE         NA          opensuse15 
      33 sles-15.2    <chr [2]>       SUSE Linux Enterprise Server 15.2    <NA>            https://suse.com           registry.suse.com/suse/sle15:15.2   2021-12-31 00:00:00 TRUE    none    TRUE         NA          opensuse152
      34 sles-15.3    <chr [2]>       SUSE Linux Enterprise Server 15.3    <NA>            https://suse.com           registry.suse.com/bci/bci-base:15.3 2022-12-31 00:00:00 TRUE    rhub    TRUE         NA          opensuse153
      35 sles-15.4    <chr [2]>       SUSE Linux Enterprise Server 15.4    <NA>            https://suse.com           registry.suse.com/bci/bci-base:15.4 2023-12-31 00:00:00 TRUE    rhub    TRUE         NA          opensuse154
      36 sles-15.5    <chr [2]>       SUSE Linux Enterprise Server 15.5    <NA>            https://suse.com           registry.suse.com/bci/bci-base:15.5 2024-12-31 00:00:00 TRUE    rhub    TRUE         NA          opensuse155
      37 sles-15.6    <chr [2]>       SUSE Linux Enterprise Server 15.6    <NA>            https://suse.com           registry.suse.com/bci/bci-base:15.6 2025-12-31 00:00:00 FALSE   posit   TRUE         NA          opensuse156
      38 ubuntu-1604  <chr [2]>       Ubuntu                       16.04   Xenial Xerus    https://ubuntu.com/        ubuntu:16.04                        2021-04-30 00:00:00 TRUE    none    TRUE         NA          xenial     
      39 ubuntu-1804  <chr [2]>       Ubuntu                       18.04   Bionic Beaver   https://ubuntu.com/        ubuntu:18.04                        2023-05-31 00:00:00 TRUE    rhub    TRUE         NA          bionic     
      40 ubuntu-2004  <chr [2]>       Ubuntu                       20.04   Focal Fossa     https://ubuntu.com/        ubuntu:20.04                        2025-05-31 00:00:00 FALSE   posit   TRUE         NA          focal      
      41 ubuntu-2204  <chr [2]>       Ubuntu                       22.04   Jammy Jellyfish https://ubuntu.com/        ubuntu:22.04                        2027-04-01 00:00:00 FALSE   posit   TRUE         NA          jammy      
      42 ubuntu-2404  <chr [2]>       Ubuntu                       24.04   Noble Numbat    https://ubuntu.com/        ubuntu:24.04                        2034-04-01 00:00:00 FALSE   posit   TRUE         TRUE        noble      

# rtools_versions

    Code
      utils::tail(rtools_versions("x86_64"))
    Output
      # A data frame: 6 x 4
        version url                                                                     first last   
      * <chr>   <chr>                                                                   <chr> <chr>  
      1 35      https://cran.rstudio.com/bin/windows/Rtools/Rtools35.exe                3.3.0 3.6.3  
      2 40      https://cran.rstudio.com/bin/windows/Rtools/rtools40-x86_64.exe         4.0.0 4.2.100
      3 42      https://github.com/r-hub/rtools42/releases/download/latest/rtools42.exe 4.2.0 4.2.100
      4 43      https://github.com/r-hub/rtools43/releases/download/latest/rtools43.exe 4.3.0 4.3.100
      5 44      https://github.com/r-hub/rtools44/releases/download/latest/rtools44.exe 4.4.0 4.4.100
      6 45      https://github.com/r-hub/rtools45/releases/download/latest/rtools45.exe 4.5.0 100.0.0
    Code
      utils::tail(rtools_versions("aarch64"))
    Output
      # A data frame: 2 x 4
        version url                                                                             first last   
      * <chr>   <chr>                                                                           <chr> <chr>  
      1 44      https://github.com/r-hub/rtools44/releases/download/latest/rtools44-aarch64.exe 4.4.0 4.4.100
      2 45      https://github.com/r-hub/rtools45/releases/download/latest/rtools45-aarch64.exe 4.5.0 100.0.0
    Code
      utils::tail(rtools_versions("arm64"))
    Output
      # A data frame: 2 x 4
        version url                                                                             first last   
      * <chr>   <chr>                                                                           <chr> <chr>  
      1 44      https://github.com/r-hub/rtools44/releases/download/latest/rtools44-aarch64.exe 4.4.0 4.4.100
      2 45      https://github.com/r-hub/rtools45/releases/download/latest/rtools45-aarch64.exe 4.5.0 100.0.0

# r_release_macos

    Code
      r_release_macos("x86_64")
    Output
      # A data frame: 1 x 5
        version date                nickname          semver     URL                                                                       
        <chr>   <dttm>              <chr>             <pckg_vrs> <chr>                                                                     
      1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1      https://cran.rstudio.com/bin/macosx/big-sur-x86_64/base/R-4.5.1-x86_64.pkg
    Code
      r_release_macos("arm64")
    Output
      # A data frame: 1 x 5
        version date                nickname          semver     URL                                                                     
        <chr>   <dttm>              <chr>             <pckg_vrs> <chr>                                                                   
      1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1      https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.5.1-arm64.pkg

---

    Code
      r_release_macos("foobar")
    Condition
      Error in `match.arg()`:
      ! 'arg' should be one of "x86_64", "arm64"

# api errors

    Code
      r_versions()
    Condition
      Error in `api()`:
      ! Failed to fetch http://127.0.0.1:<port>//r-versions
      {"error":["Internal server error"]}

# version without nickname

    Code
      resolve("2.0.0", platform = NA)
    Output
      # A data frame: 1 x 6
        version date                nickname semver     url                                                  type 
        <chr>   <dttm>              <chr>    <pckg_vrs> <chr>                                                <chr>
      1 2.0.0   2004-10-04 14:24:38 <NA>     2.0.0      https://cran.rstudio.com/src/base/R-2/R-2.0.0.tar.gz 2.0.0

# available, macos

    Code
      available("macos")
    Output
      # A data frame: 20 x 5
         version date                semver     type    url                                                                     
         <chr>   <dttm>              <pckg_vrs> <chr>   <chr>                                                                   
       1 4.1.0   2021-05-18 07:05:22 4.1.0      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.1.0-arm64.pkg
       2 4.1.1   2021-08-10 07:05:06 4.1.1      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.1.1-arm64.pkg
       3 4.1.2   2021-11-01 08:05:12 4.1.2      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.1.2-arm64.pkg
       4 4.1.3   2022-03-10 08:05:38 4.1.3      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.1.3-arm64.pkg
       5 4.2.0   2022-04-22 07:05:41 4.2.0      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.2.0-arm64.pkg
       6 4.2.1   2022-06-23 07:05:33 4.2.1      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.2.1-arm64.pkg
       7 4.2.2   2022-10-31 08:05:54 4.2.2      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.2.2-arm64.pkg
       8 4.2.3   2023-03-15 08:06:01 4.2.3      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.2.3-arm64.pkg
       9 4.3.0   2023-04-21 07:06:14 4.3.0      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.3.0-arm64.pkg
      10 4.3.1   2023-06-16 07:06:07 4.3.1      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.3.1-arm64.pkg
      11 4.3.2   2023-10-31 08:07:42 4.3.2      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.3.2-arm64.pkg
      12 4.3.3   2024-02-29 08:07:53 4.3.3      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.3.3-arm64.pkg
      13 4.4.0   2024-04-24 04:07:56 4.4.0      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.4.0-arm64.pkg
      14 4.4.1   2024-06-14 07:08:17 4.4.1      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.4.1-arm64.pkg
      15 4.4.2   2024-10-31 08:09:02 4.4.2      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.4.2-arm64.pkg
      16 4.4.3   2025-02-28 08:08:59 4.4.3      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.4.3-arm64.pkg
      17 4.5.0   2025-04-11 08:51:39 4.5.0      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.5.0-arm64.pkg
      18 4.5.1   2025-06-13 07:12:01 4.5.1      release https://cran.rstudio.com/bin/macosx/big-sur-arm64/base/R-4.5.1-arm64.pkg
      19 4.5.1   NA                  4.5.1      patched https://mac.r-project.org/big-sur/last-success/R-4.5-branch-arm64.pkg   
      20 4.6.0   NA                  4.6.0      devel   https://mac.r-project.org/big-sur/last-success/R-devel-arm64.pkg        

# available, macos-x86_64

    Code
      available("macos-x86_64")
    Output
      # A data frame: 74 x 5
         version date                semver     type    url                                                            
         <chr>   <dttm>              <pckg_vrs> <chr>   <chr>                                                          
       1 2.10.0  2009-10-26 09:02:22 2.10.0     release https://cran-archive.r-project.org/bin/macosx/base/R-2.10.0.pkg
       2 2.10.1  2009-12-14 10:28:24 2.10.1     release https://cran-archive.r-project.org/bin/macosx/base/R-2.10.1.pkg
       3 2.11.0  2010-04-22 08:11:21 2.11.0     release https://cran-archive.r-project.org/bin/macosx/base/R-2.11.0.pkg
       4 2.11.1  2010-05-31 08:10:25 2.11.1     release https://cran-archive.r-project.org/bin/macosx/base/R-2.11.1.pkg
       5 2.12.0  2010-10-15 08:41:57 2.12.0     release https://cran-archive.r-project.org/bin/macosx/base/R-2.12.0.pkg
       6 2.12.1  2010-12-16 09:12:04 2.12.1     release https://cran-archive.r-project.org/bin/macosx/base/R-2.12.1.pkg
       7 2.12.2  2011-02-25 11:07:19 2.12.2     release https://cran-archive.r-project.org/bin/macosx/base/R-2.12.2.pkg
       8 2.13.0  2011-04-13 08:31:27 2.13.0     release https://cran-archive.r-project.org/bin/macosx/base/R-2.13.0.pkg
       9 2.13.1  2011-07-08 09:37:08 2.13.1     release https://cran-archive.r-project.org/bin/macosx/base/R-2.13.1.pkg
      10 2.13.2  2011-09-30 07:05:56 2.13.2     release https://cran-archive.r-project.org/bin/macosx/base/R-2.13.2.pkg
      # i 64 more rows

# available, windows

    Code
      utils::tail(available("windows"))
    Output
      # A data frame: 6 x 5
        version date                semver     type    url                                                                
      * <chr>   <dttm>              <pckg_vrs> <chr>   <chr>                                                              
      1 4.4.2   2024-10-31 08:09:02 4.4.2      release https://cran.rstudio.com/bin/windows/base/old/4.4.2/R-4.4.2-win.exe
      2 4.4.3   2025-02-28 08:08:59 4.4.3      release https://cran.rstudio.com/bin/windows/base/old/4.4.3/R-4.4.3-win.exe
      3 4.5.0   2025-04-11 08:51:39 4.5.0      release https://cran.rstudio.com/bin/windows/base/old/4.5.0/R-4.5.0-win.exe
      4 4.5.1   2025-06-13 07:12:01 4.5.1      release https://cran.rstudio.com/bin/windows/base/old/4.5.1/R-4.5.1-win.exe
      5 4.5.1   NA                  4.5.1      patched https://cran.rstudio.com/bin/windows/base/R-4.5.1patched-win.exe   
      6 4.6.0   NA                  4.6.0      devel   https://cran.rstudio.com/bin/windows/base/R-devel-win.exe          

# available, windows-aarch64

    Code
      available("windows-aarch64")
    Output
      # A data frame: 8 x 5
        version date                semver     type    url                                                                                            
        <chr>   <dttm>              <pckg_vrs> <chr>   <chr>                                                                                          
      1 4.4.0   2024-04-24 04:07:56 4.4.0      release https://github.com/r-hub/R/releases/download/v4.4.0/R-4.4.0-aarch64.exe                        
      2 4.4.1   2024-06-14 07:08:17 4.4.1      release https://github.com/r-hub/R/releases/download/v4.4.1/R-4.4.1-aarch64.exe                        
      3 4.4.2   2024-10-31 08:09:02 4.4.2      release https://github.com/r-hub/R/releases/download/v4.4.2/R-4.4.2-aarch64.exe                        
      4 4.4.3   2025-02-28 08:08:59 4.4.3      release https://github.com/r-hub/R/releases/download/v4.4.3/R-4.4.3-aarch64.exe                        
      5 4.5.0   2025-04-11 08:51:39 4.5.0      release https://github.com/r-hub/R/releases/download/v4.5.0/R-4.5.0-aarch64.exe                        
      6 4.5.1   2025-06-13 07:12:01 4.5.1      release https://github.com/r-hub/R/releases/download/v4.5.1/R-4.5.1-aarch64.exe                        
      7 4.5.1   NA                  4.5.1      patched https://github.com/r-hub/R/releases/download/vnext/R-next-patched-2025-09-24-r88904-aarch64.exe
      8 4.6.0   NA                  4.6.0      devel   https://github.com/r-hub/R/releases/download/vdevel/R-devel-2025-10-06-r88901-aarch64.exe      

# available, ubuntu

    Code
      utils::tail(available("linux-ubuntu-24.04"))
    Output
      # A data frame: 6 x 5
        version date                semver     type    url                                                        
      * <chr>   <dttm>              <pckg_vrs> <chr>   <chr>                                                      
      1 4.4.2   2024-10-31 08:09:02 4.4.2      release https://cdn.posit.co/r/ubuntu-2404/pkgs/r-4.4.2_1_amd64.deb
      2 4.4.3   2025-02-28 08:08:59 4.4.3      release https://cdn.posit.co/r/ubuntu-2404/pkgs/r-4.4.3_1_amd64.deb
      3 4.5.0   2025-04-11 08:51:39 4.5.0      release https://cdn.posit.co/r/ubuntu-2404/pkgs/r-4.5.0_1_amd64.deb
      4 4.5.1   2025-06-13 07:12:01 4.5.1      release https://cdn.posit.co/r/ubuntu-2404/pkgs/r-4.5.1_1_amd64.deb
      5 4.5.1   NA                  4.5.1      patched https://cdn.posit.co/r/ubuntu-2404/pkgs/r-next_1_amd64.deb 
      6 4.6.0   NA                  4.6.0      devel   https://cdn.posit.co/r/ubuntu-2404/pkgs/r-devel_1_amd64.deb

# available, ubuntu aarch64

    Code
      utils::tail(available("aarch64-unknown-linux-ubuntu-24.04"))
    Output
      # A data frame: 6 x 5
        version date                semver     type    url                                                        
      * <chr>   <dttm>              <pckg_vrs> <chr>   <chr>                                                      
      1 4.4.2   2024-10-31 08:09:02 4.4.2      release https://cdn.posit.co/r/ubuntu-2404/pkgs/r-4.4.2_1_arm64.deb
      2 4.4.3   2025-02-28 08:08:59 4.4.3      release https://cdn.posit.co/r/ubuntu-2404/pkgs/r-4.4.3_1_arm64.deb
      3 4.5.0   2025-04-11 08:51:39 4.5.0      release https://cdn.posit.co/r/ubuntu-2404/pkgs/r-4.5.0_1_arm64.deb
      4 4.5.1   2025-06-13 07:12:01 4.5.1      release https://cdn.posit.co/r/ubuntu-2404/pkgs/r-4.5.1_1_arm64.deb
      5 4.5.1   NA                  4.5.1      patched https://cdn.posit.co/r/ubuntu-2404/pkgs/r-next_1_arm64.deb 
      6 4.6.0   NA                  4.6.0      devel   https://cdn.posit.co/r/ubuntu-2404/pkgs/r-devel_1_arm64.deb

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rversions.R
\name{r_versions}
\alias{r_versions}
\title{Query R's past and present versions}
\usage{
r_versions(dots = TRUE)
}
\arguments{
\item{dots}{Whether to use dots instead of dashes in the version
number.}
}
\value{
A data frame with three columns: \sQuote{version},
\sQuote{date} and \sQuote{nickname}.
}
\description{
R version numbers consist of three numbers (since version 1.4.1):
major, minor and patch.
}
\details{
We extract the version numbers from the tags in the SVN repository.
}
\examples{
\dontshow{if (rversions:::is_online()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
r_versions()
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linux-distros.R
\name{linux_distros}
\alias{linux_distros}
\title{Return information about Posit's (and some other) R builds for Linux}
\usage{
linux_distros()
}
\value{
A data frame with the following columns:
\itemize{
\item \code{id}: The distro ID, e.g. \code{ubuntu-2404}.
\item \code{aliases}: A list column of character vectors with alternative names
for the distro. Alternative names starting and ending with a forward
slash are regular expressions.
\item \code{name}: The full name of the distro, e.g. \verb{Red Hat Enterprise Linux}.
\item \code{version}: The version of the distro, e.g. \code{24.04} for Ubuntu, or \code{9}
for RHEL.
\item \code{codename}: The codename of the distro, e.g. \code{focal} for Ubuntu 20.04.
\item \code{url}: The URL of the distro's homepage.
\item \code{docker}: The name of the official Docker image for the distro,
e.g. \code{ubuntu:20.04}.
\item \code{eol}: The end-of-life date of the distro, or \code{NA} if unknown.
\item \code{retired}: whether the distro version is retired for Posit and R-hub.
\item \code{aarch64}: whether aarch64 builds are available for this distro.
Possible values are \code{posit} (builds are available from Posit),
\code{r-hub} (builds are available from R-hub), \code{none} (no builds are
available).
\item \code{ppm_binaries}: whether PPM binaries are available for this distro.
\item \code{ppm_aarch64}: whether aarch64 PPM binaries are available for this
distro.
\item \code{ppm_url}: The URL of the PPM binaries for this distro.
}
}
\description{
Return information about Posit's (and some other) R builds for Linux
}
\details{
\code{linux_distros()} caches its results in the current R session. To clear
the cache restart R or set the \code{R_VERSION_CACHE_TIMEOUT} environment
variable to \code{0}.
}
\examples{
\dontshow{if (rversions:::is_online()) withAutoprint(\{ # examplesIf}
linux_distros()
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resolve.R
\name{resolve}
\alias{resolve}
\title{Resolve an R version specification}
\usage{
resolve(version = "release", platform = current_r_platform())
}
\arguments{
\item{version}{Symbolic version name to resolve. Possible values are
\itemize{
\item \code{release}: the latest release version,
\item \code{devel}: Development version of R.
\item \code{next}: The next version of R. This is R-patched if there is no
ongoing release process currently. Otherwise it may be R-alpha,
R-beta, or R-rc or R-prerelease.
\item \code{oldrel}: The previous version of R, not counting patch versions.
\item \verb{oldrel/<n>}: The n-th previous version of R, not counting patch
versions.
\item \code{x.y.z}: A specific R version, e.g. 4.5.1.
\item \code{x.y}: The last patch release from a minor R branch, e.g. 4.4.
}}

\item{platform}{Operating system version. Defaults to
\code{\link[=current_r_platform]{current_r_platform()}}. Possible values are:
\itemize{
\item \code{NA}: to get information about platform-independent release archives.
\item \code{windows}: x86_64 Windows.
\item \code{windows-x86_64}: the same.
\item \code{windows-aarch64}: aarch64 Windows.
\item \code{macos}: arm64 macOS.
\item \code{macos-arm64}: the same.
\item \code{macos-x86_64}: x86_64 macOS.
\item \verb{linux-<distro>-<release>}: x86_64 Linux on the given distribution
and release, e.g. \code{linux-ubuntu-22.04}.
\item A target triplet of the form \code{cpu-vendor-os-abi}. On Linux it should
also include the distribution name and release. See
\code{\link[=current_r_platform]{current_r_platform()}} for examples.
}}
}
\value{
A one row data frame, with columns \code{version}, \code{date},
\code{nickname}, \code{semver}, \code{url}, \code{type}, \code{rtools_url} (only for Windows).
}
\description{
Resolve an R version specification
}
\details{
\code{resolve()} caches its results in the current R session. To clear the
cache restart R or set the \code{R_VERSION_CACHE_TIMEOUT} environment
variable to \code{0}.
}
\examples{
\dontshow{if (rversions:::is_online()) withAutoprint(\{ # examplesIf}
resolve("release", "windows")
resolve("4.4", "windows-aarch64")
resolve("oldrel", "macos")
resolve("devel", "linux-gnu-ubuntu-24.04")
resolve("devel", "aarch64-unknown-linux-gnu-ubuntu-22.04")
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rversions-package.R
\docType{package}
\name{rversions-package}
\alias{rversions}
\alias{rversions-package}
\title{rversions: Query 'R' Versions, Including 'r-release' and 'r-oldrel'}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Query the main 'R' 'SVN' repository to find the versions 'r-release' and 'r-oldrel' refer to, and also all previous 'R' versions and their release dates.
}
\details{
The R version numbers and dates are extracted from the
main R SVN repository at \url{https://svn.r-project.org/R/}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/r-hub/rversions}
  \item \url{https://r-hub.github.io/rversions/}
  \item Report bugs at \url{https://github.com/r-hub/rversions/issues}
}

}
\author{
\strong{Maintainer}: Gábor Csárdi \email{csardi.gabor@gmail.com}

Other contributors:
\itemize{
  \item Jeroen Ooms \email{jeroen.ooms@stat.ucla.edu} [contributor]
  \item R Consortium [funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tarball.R
\name{r_release_tarball}
\alias{r_release_tarball}
\title{Latest release for which a source tarball is available for download}
\usage{
r_release_tarball()
}
\value{
A one row data frame, with columns \sQuote{version},
\sQuote{date}, \sQuote{URL} and \sQuote{nickname}.
}
\description{
\code{\link[=r_release]{r_release()}} works based on the SVN tags in the main R source code
repository. However, an SVN tag does not mean that the same version
is available for download. \code{r_release_tarball} returns the latest
version for which a source tarball is available.
}
\examples{
r_release()
r_release_tarball()
}
\seealso{
Other R downloads: 
\code{\link{r_release_macos}()},
\code{\link{r_release_win}()}
}
\concept{R downloads}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/available.R
\name{available}
\alias{available}
\title{Available R Builds}
\usage{
available(platform = current_r_platform())
}
\arguments{
\item{platform}{Operating system version. Defaults to
\code{\link[=current_r_platform]{current_r_platform()}}. Possible values are:
\itemize{
\item \code{windows}: x86_64 Windows.
\item \code{windows-x86_64}: the same.
\item \code{windows-aarch64}: aarch64 Windows.
\item \code{macos}: arm64 macOS.
\item \code{macos-arm64}: the same.
\item \code{macos-x86_64}: x86_64 macOS.
\item \verb{linux-<distro>-<release>}: x86_64 Linux on the given distribution
and release, e.g. \code{linux-ubuntu-22.04}.
\item A target triplet of the form \code{cpu-vendor-os-abi}. On Linux it should
also include the distribution name and release. See
\code{\link[=current_r_platform]{current_r_platform()}} for examples.
}}
}
\value{
A data frame with columns \code{version}, \code{date}, \code{semver}, \code{type},
and \code{url}.
}
\description{
Available R Builds
}
\details{
\code{available()} caches its results in the current R session. To clear the
cache restart R or set the \code{R_VERSION_CACHE_TIMEOUT} environment
variable to \code{0}.
}
\examples{
\dontshow{if (rversions:::is_online()) withAutoprint(\{ # examplesIf}
available()
available("windows")
available("macos-arm64")
available("linux-gnu-ubuntu-24.04")
available("aarch64-unknown-linux-gnu-ubuntu-22.04")
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/macos.R
\name{r_release_macos}
\alias{r_release_macos}
\title{Latest R release with a macOS installer available for download}
\usage{
r_release_macos()
}
\value{
A one row data frame, with columns \sQuote{version},
\sQuote{date}, \sQuote{URL} and \sQuote{nickname}.
}
\description{
\code{\link[=r_release]{r_release()}} works based on the SVN tags in the main R source code
repository. However, an SVN tag does not mean that the same version
is available for download. \code{r_release_macos} returns the latest
version for which a macOS installer is available.
}
\examples{
r_release()
r_release_macos()
}
\seealso{
Other R downloads: 
\code{\link{r_release_tarball}()},
\code{\link{r_release_win}()}
}
\concept{R downloads}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/win.R
\name{r_release_win}
\alias{r_release_win}
\title{Latest R release with a Windows installer available for download}
\usage{
r_release_win()
}
\value{
A one row data frame, with columns \sQuote{version},
\sQuote{date}, \sQuote{URL} and \sQuote{nickname}.
}
\description{
\code{\link[=r_release]{r_release()}} works based on the SVN tags in the main R source code
repository. However, an SVN tag does not mean that the same version
is available for download. \code{r_release_win} returns the latest
version for which a Windows installer is available.
}
\section{Note}{

This function is superseded by \code{\link[=resolve]{resolve()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{r_release_win()
#> # A data frame: 1 x 5
#>   version date                nickname          semver     URL                  
#>   <chr>   <dttm>              <chr>             <pckg_vrs> <chr>                
#> 1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1      https://cran.rstudio~
}\if{html}{\out{</div>}}

\if{html}{\out{<div class="sourceCode r">}}\preformatted{resolve("release", platform = "windows")
#> # A data frame: 1 x 7
#>   version date                nickname          semver    url   type  rtools_url
#>   <chr>   <dttm>              <chr>             <pckg_vr> <chr> <chr> <chr>     
#> 1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1     http~ rele~ https://g~
}\if{html}{\out{</div>}}
}

\examples{
\dontshow{if (rversions:::is_online()) withAutoprint(\{ # examplesIf}
r_release()
r_release_win()
\dontshow{\}) # examplesIf}
}
\seealso{
Other R downloads: 
\code{\link{r_release_macos}()},
\code{\link{r_release_tarball}()}
}
\concept{R downloads}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rversions.R
\name{r_release}
\alias{r_release}
\title{Version number of R-release}
\usage{
r_release(dots = TRUE)
}
\arguments{
\item{dots}{Whether to use dots instead of dashes in the version
number.}
}
\value{
A one row data frame, with columns \sQuote{version},
\sQuote{date} and \sQuote{nickname}.
}
\description{
The latest tag in the SVN repository (in terms of version numbers,
not dates).
}
\section{Note}{

This function is superseded by \code{\link[=resolve]{resolve()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{r_release()
#> # A data frame: 1 x 6
#>   version date                nickname          semver     URL             type 
#>   <chr>   <dttm>              <chr>             <pckg_vrs> <chr>           <chr>
#> 1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1      https://cran.r~ rele~
}\if{html}{\out{</div>}}

\if{html}{\out{<div class="sourceCode r">}}\preformatted{resolve("release", platform = NA)
#> # A data frame: 1 x 6
#>   version date                nickname          semver     url             type 
#>   <chr>   <dttm>              <chr>             <pckg_vrs> <chr>           <chr>
#> 1 4.5.1   2025-06-13 07:12:01 Great Square Root 4.5.1      https://cran.r~ rele~
}\if{html}{\out{</div>}}
}

\examples{
\dontshow{if (rversions:::is_online()) withAutoprint(\{ # examplesIf}
r_release()
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/platform.R
\name{current_r_platform}
\alias{current_r_platform}
\title{Current R platform}
\usage{
current_r_platform()
}
\description{
\code{current_r_platform()} detects the platform of the current R version.
}
\details{
It returns a string of the form \code{cpu-vendor-os-abi}, a target triplet,
see some examples below. On Linux it also includes the distribution
name and version and part of the ABI.

Examples:
\itemize{
\item \code{x86_64-apple-darwin17.0}: macOS High Sierra.
\item \code{aarch64-apple-darwin20}: macOS Big Sur on arm64.
\item \code{x86_64-w64-mingw32}: 64 bit Windows.
\item \code{i386-w64-mingw32}: 32 bit Windows.
\item \code{i386+x86_64-w64-mingw32}: 64 bit + 32 bit Windows.
\item \code{i386-pc-solaris2.10}: 32 bit Solaris. (Some broken 64 Solaris
builds might have the same platform string, unfortunately.)
\item \code{x86_64-pc-linux-gnu-debian-10}: Debian Linux 10 on x86_64.
\item \verb{x86_64-pc-linux-musl-alpine-3.14.1}: Alpine Linux.
\item \code{x86_64-pc-linux-gnu-unknown}: Unknown Linux Distribution on x86_64.
\item \code{s390x-ibm-linux-gnu-ubuntu-20.04}: Ubuntu Linux 20.04 on S390x.
\item \code{amd64-portbld-freebsd12.1}: FreeBSD 12.1 on x86_64.
}
}
\examples{
current_r_platform()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rtools-versions.R
\name{rtools_versions}
\alias{rtools_versions}
\title{List Rtools versions}
\usage{
rtools_versions(arch = NULL)
}
\arguments{
\item{arch}{Architecture. Possible values are:
\itemize{
\item \code{x86_64}: x86_64 Windows,
\item \code{aarch64} or \code{arm64}: aarch64 Windows.
Defaults tp the current architecture on Windows, and to \code{x86_64} on
other systems.
}}
}
\value{
A data frame with columns
\itemize{
\item \code{version}: The Rtools version, e.g. \code{45}.
\item \code{url}: Download URL of the Rtools installer.
\item \code{first}: First R version supported by this Rtools version.
\item \code{last}: Last R version supported by this Rtools version.
}
}
\description{
List Rtools versions
}
\examples{
\dontshow{if (rversions:::is_online()) withAutoprint(\{ # examplesIf}
rtools_versions()
rtools_versions("aarch64")
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rversions.R
\name{r_oldrel}
\alias{r_oldrel}
\title{Version number of R-oldrel}
\usage{
r_oldrel(dots = TRUE)
}
\arguments{
\item{dots}{Whether to use dots instead of dashes in the version
number.}
}
\value{
A one row data frame, with columns \sQuote{version},
\sQuote{date} and \sQuote{nickname}.
}
\description{
R-oldrel is the latest version of the previous minor version.
We extract version numbers from the R SVN repository tags.
}
\section{Note}{

This function is superseded by \code{\link[=resolve]{resolve()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{r_oldrel()
#> # A data frame: 1 x 5
#>   version date                nickname    semver     URL                        
#>   <chr>   <dttm>              <chr>       <pckg_vrs> <chr>                      
#> 1 4.4.3   2025-02-28 08:08:59 Trophy Case 4.4.3      https://cran.rstudio.com/s~
}\if{html}{\out{</div>}}

\if{html}{\out{<div class="sourceCode r">}}\preformatted{resolve("oldrel", platform = NA)
#> # A data frame: 1 x 6
#>   version date                nickname    semver     url                   type 
#>   <chr>   <dttm>              <chr>       <pckg_vrs> <chr>                 <chr>
#> 1 4.4.3   2025-02-28 08:08:59 Trophy Case 4.4.3      https://cran.rstudio~ oldr~
}\if{html}{\out{</div>}}
}

\examples{
\dontshow{if (rversions:::is_online()) withAutoprint(\{ # examplesIf}
r_oldrel()
\dontshow{\}) # examplesIf}
}
